@@ -20,19 +20,21 @@
 
 """Module for implementing dialog box for viewing/editing data."""
 
-import veusz.qtall as qt4
+from __future__ import division
 
-import veusz.document as document
-from veusz.qtwidgets.datasetbrowser import DatasetBrowser
-from veuszdialog import VeuszDialog
+from ..compat import cstr
+from .. import qtall as qt4
+from .. import document
+from .. import setting
+from ..qtwidgets.datasetbrowser import DatasetBrowser
+from .veuszdialog import VeuszDialog
 
 # register function to dataset class to edit dataset
 recreate_register = {}
 
 def _(text, disambiguation=None, context="DataEditDialog"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class DatasetTableModel1D(qt4.QAbstractTableModel):
     """Provides access to editing and viewing of datasets."""
@@ -42,8 +44,7 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
 
         self.document = document
         self.dsname = datasetname
-        self.connect(document, qt4.SIGNAL('sigModified'),
-                     self.slotDocumentModified)
+        document.signalModified.connect(self.slotDocumentModified)
 
     def rowCount(self, parent):
         """Return number of rows."""
@@ -78,17 +79,18 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
         if ds is not None:
             # select correct part of dataset
             data = getattr(ds, ds.columns[index.column()])
-        if ds is not None and data is not None and role == qt4.Qt.DisplayRole:
+        if ds is not None and data is not None and role in (
+            qt4.Qt.DisplayRole, qt4.Qt.EditRole):
             # blank row at end of data
             if index.row() == len(data):
-                return qt4.QVariant()
+                return None
 
-            # convert data to QVariant
+            # convert data to data
             d = data[index.row()]
-            return ds.uiDataItemToQVariant(d)
+            return ds.uiDataItemToData(d)
 
         # empty entry
-        return qt4.QVariant()
+        return None
 
     def headerData(self, section, orientation, role):
         """Return row numbers or column names."""
@@ -96,25 +98,28 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
         try:
             ds = self.document.data[self.dsname]
         except KeyError:
-            return qt4.QVariant()
+            return None
 
         if role == qt4.Qt.DisplayRole:
             if orientation == qt4.Qt.Horizontal:
                 # column names
-                return qt4.QVariant( ds.column_descriptions[section] )
+                return ds.column_descriptions[section]
             else:
                 if section == len(ds.data):
                     return "+"
                 # return row numbers
-                return qt4.QVariant(section+1)
+                return section+1
+
+        return None
 
-        return qt4.QVariant()
-        
     def flags(self, index):
         """Update flags to say that items are editable."""
         if index.isValid():
-            return ( qt4.QAbstractTableModel.flags(self, index) |
-                     qt4.Qt.ItemIsEditable )
+            f = qt4.QAbstractTableModel.flags(self, index)
+            ds = self.document.data.get(self.dsname)
+            if ds is not None and ds.editable():
+                f |= qt4.Qt.ItemIsEditable
+            return f
         return qt4.Qt.ItemIsEnabled
 
     def removeRows(self, row, count):
@@ -152,7 +157,7 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
 
         # update if conversion okay
         try:
-            val = ds.uiConvertToDataItem( value.toString() )
+            val = ds.uiConvertToDataItem(value)
         except ValueError:
             return False
 
@@ -174,8 +179,7 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
 
         self.document = document
         self.dsnames = datasetnames
-        self.connect(document, qt4.SIGNAL('sigModified'),
-                     self.slotDocumentModified)
+        document.signalModified.connect(self.slotDocumentModified)
 
         self.changeset = -1
         self.rows = 0
@@ -240,12 +244,12 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
 
         if role == qt4.Qt.DisplayRole:
             if index.row() < self.rowcounts[dsidx]-1:
-                # convert data to QVariant
+                # convert data to Data
                 d = data[index.row()]
-                return ds.uiDataItemToQVariant(d)
+                return ds.uiDataItemToData(d)
 
         # empty entry
-        return qt4.QVariant()
+        return None
 
     def headerData(self, section, orientation, role):
         """Return row numbers or column names."""
@@ -257,20 +261,24 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
                 ds = self.document.data[dsname]
                 descr = ds.column_descriptions[colidx]
                 header = dsname + '\n' + descr
-                return qt4.QVariant(header)
+                return header
             else:
                 # return row numbers
                 if section == self.rows-1:
-                    return qt4.QVariant("+")
-                return qt4.QVariant(section+1)
+                    return "+"
+                return section+1
 
-        return qt4.QVariant()
+        return None
 
     def flags(self, index):
         """Update flags to say that items are editable."""
         if index.isValid():
-            return ( qt4.QAbstractTableModel.flags(self, index) |
-                     qt4.Qt.ItemIsEditable )
+            f = qt4.QAbstractTableModel.flags(self, index)
+            dsname, colname, dsidx, colidx = self.colattrs[index.column()]
+            ds = self.document.data.get(dsname)
+            if ds is not None and ds.editable():
+                f |= qt4.Qt.ItemIsEditable
+            return f
         return qt4.Qt.ItemIsEnabled
 
     def setData(self, index, value, role):
@@ -294,7 +302,7 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
 
         # convert text to value
         try:
-            val = ds.uiConvertToDataItem( value.toString() )
+            val = ds.uiConvertToDataItem(value)
         except ValueError:
             return False
 
@@ -333,8 +341,7 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
 
         self.document = document
         self.dsname = datasetname
-        self.connect(document, qt4.SIGNAL('sigModified'),
-                     self.slotDocumentModified)
+        document.signalModified.connect(self.slotDocumentModified)
 
     def rowCount(self, parent):
         if parent.isValid():
@@ -360,9 +367,9 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
             ds = self.document.data[self.dsname].data
             if ds is not None:
                 num = ds[ds.shape[0]-index.row()-1, index.column()]
-                return qt4.QVariant( float(num) )
+                return float(num)
 
-        return qt4.QVariant()
+        return None
 
     def headerData(self, section, orientation, role):
         """Return headers at top."""
@@ -380,16 +387,20 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
                     r = (r[1], r[0]) # swap (as y reversed)
                     num = ds.data.shape[0]
                 val = (r[1]-r[0])/num*(section+0.5)+r[0]
-                return qt4.QVariant( '%g' % val )
+                return '%g' % val
 
-        return qt4.QVariant()
+        return None
     
     def flags(self, index):
         """Update flags to say that items are editable."""
         if not index.isValid():
             return qt4.Qt.ItemIsEnabled
         else:
-            return qt4.QAbstractTableModel.flags(self, index) | qt4.Qt.ItemIsEditable
+            f = qt4.QAbstractTableModel.flags(self, index)
+            ds = self.document.data.get(self.dsname)
+            if ds is not None and ds.editable():
+                f |= qt4.Qt.ItemIsEditable
+            return f
 
     def slotDocumentModified(self):
         """Called when document modified."""
@@ -407,7 +418,7 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
 
         # update if conversion okay
         try:
-            val = ds.uiConvertToDataItem( value.toString() )
+            val = ds.uiConvertToDataItem(value)
         except ValueError:
             return False
 
@@ -416,6 +427,26 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
         self.document.applyOperation(op)
         return True
 
+class ViewDelegate(qt4.QStyledItemDelegate):
+    """Delegate for fixing double editing.
+    Normal editing uses double spin box, which is inappropriate
+    """
+
+    def createEditor(self, parent, option, index):
+        if type(index.data()) is float:
+            return qt4.QLineEdit(parent)
+        else:
+            return qt4.QStyledItemDelegate.createEditor(
+                self, parent, option, index)
+
+    def setEditorData(self, editor, index):
+        """Override setData to use correct formatting."""
+        if type(index.data()) is float:
+            txt = setting.uilocale.toString(index.data(), 'g', 8)
+            editor.setText(txt)
+        else:
+            qt4.QStyledItemDelegate.setEditorData(self, editor, index)
+
 class DataEditDialog(VeuszDialog):
     """Dialog for editing and rearranging data sets."""
     
@@ -429,6 +460,9 @@ class DataEditDialog(VeuszDialog):
             _('Select multiple datasets to edit simultaneously'))
         self.splitter.insertWidget(0, self.dsbrowser)
 
+        self.deligate = ViewDelegate()
+        self.datatableview.setItemDelegate(self.deligate)
+
         # actions for data table
         for text, slot in (
             (_('Copy'), self.slotCopy),
@@ -449,8 +483,7 @@ class DataEditDialog(VeuszDialog):
         self.linkedlabel.viewport().setBackgroundRole(qt4.QPalette.Window)
 
         # document changes
-        self.connect(document, qt4.SIGNAL('sigModified'),
-                     self.slotDocumentModified)
+        document.signalModified.connect(self.slotDocumentModified)
 
         # select first item, if any or initialise if none
         if len(self.document.data) > 0:
@@ -610,8 +643,8 @@ class DataEditDialog(VeuszDialog):
                     lines.append( '\t'.join(rowitems) )
                     rowitems = []
                 lastrow = row
-            rowitems.append( unicode(
-                model.createIndex(row, column).data().toString()) )
+            rowitems.append(
+                cstr(model.createIndex(row, column).data()) )
         if rowitems:
             lines.append( '\t'.join(rowitems) )
         lines.append('')  # blank line at end
