@@ -20,23 +20,24 @@
 
 """Module for implementing dialog boxes for importing data in Veusz."""
 
+from __future__ import division
 import os.path
 import re
 import csv
 import sys
 
-import veusz.qtall as qt4
-import veusz.document as document
-import veusz.setting as setting
-import veusz.utils as utils
-import veusz.plugins as plugins
-import exceptiondialog
-from veuszdialog import VeuszDialog
+from ..compat import crange, czip, citems, cnext, cstr
+from .. import qtall as qt4
+from .. import document
+from .. import setting
+from .. import utils
+from .. import plugins
+from . import exceptiondialog
+from .veuszdialog import VeuszDialog
 
 def _(text, disambiguation=None, context="ImportDialog"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class ImportTab(qt4.QWidget):
     """Tab for a particular import type."""
@@ -51,7 +52,7 @@ class ImportTab(qt4.QWidget):
 
     def loadUi(self):
         """Load up UI file."""
-        qt4.loadUi(os.path.join(utils.veuszDirectory, 'dialogs',
+        qt4.loadUi(os.path.join(utils.resourceDirectory, 'ui',
                                 self.resource), self)
         self.uiloaded = True
 
@@ -125,7 +126,7 @@ class ImportTabStandard(ImportTab):
         """Standard Veusz importing."""
 
         # convert controls to values
-        descriptor = unicode( self.descriptoredit.text() )
+        descriptor = self.descriptoredit.text()
         useblocks = self.blockcheckbox.isChecked()
         ignoretext = self.ignoretextcheckbox.isChecked()
 
@@ -155,7 +156,7 @@ class ImportTabStandard(ImportTab):
         # tell the user what happened
         # failures in conversion
         lines = []
-        for var, count in op.outinvalids.iteritems():
+        for var, count in citems(op.outinvalids):
             if count != 0:
                 lines.append(_('%i conversions failed for dataset "%s"') %
                              (count, var))
@@ -247,16 +248,18 @@ class ImportTabCSV(ImportTab):
             return False
 
         try:
-            reader = utils.UnicodeCSVReader( filename,
-                                             delimiter=delimiter,
-                                             quotechar=textdelimiter,
-                                             encoding=encoding )
+            reader = utils.get_unicode_csv_reader(
+                filename,
+                delimiter=delimiter,
+                quotechar=textdelimiter,
+                encoding=encoding )
+
             # construct list of rows
             rows = []
             numcols = 0
             try:
-                for i in xrange(10):
-                    row = reader.next()
+                for i in crange(10):
+                    row = cnext(reader)
                     rows.append(row)
                     numcols = max(numcols, len(row))
                 rows.append(['...'])
@@ -271,10 +274,10 @@ class ImportTabCSV(ImportTab):
         # fill up table
         t.setColumnCount(numcols)
         t.setRowCount(numrows)
-        for r in xrange(numrows):
-            for c in xrange(numcols):
+        for r in crange(numrows):
+            for c in crange(numcols):
                 if c < len(rows[r]):
-                    item = qt4.QTableWidgetItem(unicode(rows[r][c]))
+                    item = qt4.QTableWidgetItem(rows[r][c])
                     t.setItem(r, c, item)
 
         return True
@@ -297,7 +300,7 @@ class ImportTabCSV(ImportTab):
         headerignore = self.csvignorehdrspin.value()
         rowsignore = self.csvignoretopspin.value()
         blanksaredata = self.csvblanksdatacheck.isChecked()
-        dateformat = unicode(self.csvdatefmtcombo.currentText())
+        dateformat = self.csvdatefmtcombo.currentText()
         headermode = ('multi', '1st', 'none')[
             self.csvheadermodecombo.currentIndex()]
 
@@ -368,7 +371,7 @@ class ImportTab2D(ImportTab):
 
     def doPreview(self, filename, encoding):
         """Preview 2d dataset files."""
-        
+
         try:
             ifile = utils.openEncoding(filename, encoding)
             text = ifile.read(4096)+'\n'
@@ -388,7 +391,7 @@ class ImportTab2D(ImportTab):
         # this really needs improvement...
 
         # get datasets and split into a list
-        datasets = unicode( self.twod_datasetsedit.text() )
+        datasets = self.twod_datasetsedit.text()
         datasets = re.split('[, ]+', datasets)
 
         # strip out blank items
@@ -399,12 +402,12 @@ class ImportTab2D(ImportTab):
             self.twod_previewedit.setPlainText(_('At least one dataset needs to '
                                                  'be specified'))
             return
-        
+
         # convert range parameters
         ranges = []
         for e in (self.twod_xminedit, self.twod_xmaxedit,
                   self.twod_yminedit, self.twod_ymaxedit):
-            f = unicode(e.text())
+            f = e.text()
             r = None
             try:
                 r = float(f)
@@ -446,9 +449,9 @@ class ImportTab2D(ImportTab):
             for ds in op.outdatasets:
                 output.append(' %s' % doc.data[ds].description(
                         showlinked=False))
-            
+
             output = '\n'.join(output)
-        except document.Read2DError, e:
+        except document.Read2DError as e:
             output = _('Error importing datasets:\n %s') % str(e)
 
         # show status in preview box
@@ -471,6 +474,12 @@ class ImportTabFITS(ImportTab):
         self.connect( self.fitsdatacolumn,
                       qt4.SIGNAL('currentIndexChanged(int)'),
                       self.dialog.enableDisableImport )
+        self.connect( self.dialog.prefixcombo,
+                      qt4.SIGNAL('editTextChanged(const QString&)'),
+                      self.dialog.enableDisableImport )
+        self.connect( self.dialog.suffixcombo,
+                      qt4.SIGNAL('editTextChanged(const QString&)'),
+                      self.dialog.enableDisableImport )
 
         self.fitswcsmode.defaultlist = [
             _('Pixel (simple)'), _('Pixel (WCS)'), _('Fractional'),
@@ -484,7 +493,7 @@ class ImportTabFITS(ImportTab):
             cntrl = getattr(self, 'fits%scolumn' % c)
             cntrl.setCurrentIndex(0)
         self.fitswcsmode.setCurrentIndex(0)
-            
+
     def doPreview(self, filename, encoding):
         """Set up controls for FITS file."""
 
@@ -492,28 +501,28 @@ class ImportTabFITS(ImportTab):
         global pyfits
         if pyfits is None:
             try:
-                import pyfits as PF
+                from astropy.io import fits as PF
                 pyfits = PF
             except ImportError:
-                pyfits = None
+                try:
+                    import pyfits as PF
+                    pyfits = PF
+                except ImportError:
+                    pyfits = None
 
         # if it isn't
         if pyfits is None:
             self.fitslabel.setText(
-                _('FITS file support requires that PyFITS is installed.'
-                  ' You can download it from'
-                  ' http://www.stsci.edu/resources/software_hardware/pyfits'))
+                _('FITS file support requires that astropy or PyFITS '
+                  'are installed'))
             return False
-        
+
         # try to identify fits file
         try:
-            ifile = open(filename,  'rU')
-            line = ifile.readline()
-            # is this a hack?
-            if line.find('SIMPLE  =                    T') == -1:
-                raise EnvironmentError
-            ifile.close()
-        except EnvironmentError:
+            f = pyfits.open(filename)
+            f[0].header
+            f.close()
+        except Exception:
             self.clearFITSView()
             return False
 
@@ -558,7 +567,7 @@ class ImportTabFITS(ImportTab):
                 else:
                     data = ['invalidimage']
                 dims = [ str(header['NAXIS%i' % (i+1)])
-                         for i in xrange(naxis) ]
+                         for i in crange(naxis) ]
                 dims = '*'.join(dims)
                 if dims:
                     dims = '(%s)' % dims
@@ -574,7 +583,7 @@ class ImportTabFITS(ImportTab):
 
     def slotFitsUpdateCombos(self):
         """Update list of fits columns when new item is selected."""
-        
+
         items = self.fitshdulist.selectedItems()
         if len(items) != 0:
             item = items[0]
@@ -592,7 +601,7 @@ class ImportTabFITS(ImportTab):
                 cols = ['None']
                 cols += ['%s (%s)' %
                          (i.name, i.format) for i in data[1]]
-        
+
         for c in ('data', 'sym', 'pos', 'neg'):
             cntrl = getattr(self, 'fits%scolumn' % c)
             cntrl.setEnabled(enablecolumns)
@@ -610,25 +619,29 @@ class ImportTabFITS(ImportTab):
             hdunum = int( str(item.text(0)) )
 
             # any name for the dataset?
-            if not unicode(self.fitsdatasetname.text()):
+            filename = self.dialog.filenameedit.text()
+            prefix, suffix = self.dialog.getPrefixSuffix(filename)
+
+            if not ( prefix + self.fitsdatasetname.text() +
+                     suffix ).strip():
                 return False
 
             # if a table, need selected item
             data = self.fitsitemdata[hdunum]
             if data[0] != 'image' and self.fitsdatacolumn.currentIndex() == 0:
                 return False
-            
+
             return True
         return False
 
     def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Import fits file."""
-        
+
         item = self.fitshdulist.selectedItems()[0]
         hdunum = int( str(item.text(0)) )
         data = self.fitsitemdata[hdunum]
 
-        name = prefix + unicode(self.fitsdatasetname.text()) + suffix
+        name = (prefix + self.fitsdatasetname.text() + suffix).strip()
 
         wcsmode = None
         if data[0] == 'table':
@@ -697,7 +710,7 @@ class ImportTabPlugins(ImportTab):
         ImportTab.loadUi(self)
 
         # fill plugin combo
-        names = list(sorted([p.name for p in plugins.importpluginregistry]))
+        names = sorted([p.name for p in plugins.importpluginregistry])
         self.pluginType.addItems(names)
 
         self.connect(self.pluginType, qt4.SIGNAL('currentIndexChanged(int)'),
@@ -721,20 +734,20 @@ class ImportTabPlugins(ImportTab):
 
         if idx >= 0:
             self.pluginType.setCurrentIndex(idx)
-        
+
         self.pluginChanged(-1)
 
     def getPluginFields(self):
         """Return a dict of the fields given."""
         results = {}
         plugin = self.getSelectedPlugin()
-        for field, cntrls in zip(plugin.fields, self.fields):
+        for field, cntrls in czip(plugin.fields, self.fields):
             results[field.name] = field.getControlResults(cntrls)
         return results
 
     def getSelectedPlugin(self):
         """Get instance selected plugin or none."""
-        selname = unicode(self.pluginType.currentText())
+        selname = self.pluginType.currentText()
         names = [p.name for p in plugins.importpluginregistry]
         try:
             idx = names.index(selname)
@@ -803,17 +816,17 @@ class ImportTabPlugins(ImportTab):
                                             self.getPluginFields())
         try:
             text, ok = plugin.getPreview(params)
-        except plugins.ImportPluginException, ex:
-            text = unicode(ex)
+        except plugins.ImportPluginException as ex:
+            text = cstr(ex)
             ok = False
         self.pluginPreview.setPlainText(text)
         return bool(ok)
 
     def doImport(self, doc, filename, linked, encoding, prefix, suffix, tags):
         """Import using plugin."""
-        
+
         fields = self.getPluginFields()
-        plugin = unicode(self.pluginType.currentText())
+        plugin = self.pluginType.currentText()
 
         params = document.ImportParamsPlugin(
             plugin=plugin,
@@ -826,8 +839,8 @@ class ImportTabPlugins(ImportTab):
         op = document.OperationDataImportPlugin(params)
         try:
             doc.applyOperation(op)
-        except plugins.ImportPluginException, ex:
-            self.pluginPreview.setPlainText( unicode(ex) )
+        except plugins.ImportPluginException as ex:
+            self.pluginPreview.setPlainText( cstr(ex) )
             return
 
         out = [_('Imported data for datasets:')]
@@ -961,7 +974,7 @@ class ImportDialog(VeuszDialog):
         fd.setFileMode( qt4.QFileDialog.ExistingFile )
 
         # use filename to guess a path if possible
-        filename = unicode(self.filenameedit.text())
+        filename = self.filenameedit.text()
         if os.path.isdir(filename):
             ImportDialog.dirname = filename
         elif os.path.isdir( os.path.dirname(filename) ):
@@ -977,7 +990,7 @@ class ImportDialog(VeuszDialog):
 
     def guessImportTab(self):
         """Guess import tab based on filename."""
-        filename = unicode( self.filenameedit.text() )
+        filename = self.filenameedit.text()
 
         fname, ftype = os.path.splitext(filename)
         # strip off any gz, bz2 extensions to get real extension
@@ -988,7 +1001,7 @@ class ImportDialog(VeuszDialog):
         # examine from left to right
         # promoted plugins come after plugins
         idx = -1
-        for i in xrange(self.methodtab.count()):
+        for i in crange(self.methodtab.count()):
             w = self.methodtab.widget(i)
             if w.isFiletypeSupported(ftype):
                 idx = i
@@ -1003,7 +1016,7 @@ class ImportDialog(VeuszDialog):
         # save so we can restore later
         tab = self.methodtab.currentIndex()
         setting.settingdb['import_lasttab'] = tab
-        filename = unicode(self.filenameedit.text())
+        filename = self.filenameedit.text()
         encoding = str(self.encodingcombo.currentText())
         importtab = self.methodtab.currentWidget()
 
@@ -1031,7 +1044,7 @@ class ImportDialog(VeuszDialog):
     def slotImport(self):
         """Do the importing"""
 
-        filename = unicode( self.filenameedit.text() )
+        filename = self.filenameedit.text()
         linked = self.linkcheckbox.isChecked()
         encoding = str(self.encodingcombo.currentText())
         if filename == '{clipboard}':
@@ -1043,7 +1056,7 @@ class ImportDialog(VeuszDialog):
         # import according to tab selected
         importtab = self.methodtab.currentWidget()
         prefix, suffix = self.getPrefixSuffix(filename)
-        tags = unicode(self.tagcombo.currentText()).split()
+        tags = self.tagcombo.currentText().split()
 
         try:
             qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
@@ -1061,7 +1074,7 @@ class ImportDialog(VeuszDialog):
 
     def retnDatasetInfo(self, dsnames, linked, filename):
         """Return a list of information for the dataset names given."""
-        
+
         lines = [_('Imported data for datasets:')]
         dsnames.sort()
         for name in dsnames:
@@ -1079,9 +1092,9 @@ class ImportDialog(VeuszDialog):
     def getPrefixSuffix(self, filename):
         """Get prefix and suffix values."""
         f = utils.cleanDatasetName( os.path.basename(filename) )
-        prefix = unicode( self.prefixcombo.lineEdit().text() )
+        prefix = self.prefixcombo.lineEdit().text()
         prefix = prefix.replace('$FILENAME', f)
-        suffix = unicode( self.suffixcombo.lineEdit().text() )
+        suffix = self.suffixcombo.lineEdit().text()
         suffix = suffix.replace('$FILENAME', f)
         return prefix, suffix
 
@@ -1105,6 +1118,6 @@ class ImportDialog(VeuszDialog):
     def updateClipPreview(self):
         """Clipboard contents changed, so update preview if showing clipboard."""
 
-        filename = unicode(self.filenameedit.text())
+        filename = self.filenameedit.text()
         if filename == '{clipboard}':
             self.slotUpdatePreview()
