@@ -24,21 +24,23 @@ Module supplies the command interface used in the program, and for
 external programs.
 """
 
+from __future__ import division, print_function
 import os.path
 import traceback
 
-import veusz.qtall as qt4
-import veusz.setting as setting
-import veusz.embed as embed
-import veusz.plugins as plugins
-import veusz.utils as utils
+from ..compat import citems, ckeys, cbasestr, cstr
+from .. import qtall as qt4
+from .. import setting
+from .. import embed
+from .. import plugins
+from .. import utils
 
-import importparams
-import datasets
-import operations
-import dataset_histo
-import mime
-import export
+from . import importparams
+from . import datasets
+from . import operations
+from . import dataset_histo
+from . import mime
+from . import export
 
 class CommandInterface(qt4.QObject):
     """Class provides command interface."""
@@ -152,7 +154,7 @@ class CommandInterface(qt4.QObject):
         w = self.document.applyOperation(op)
 
         if self.verbose:
-            print "Added a widget of type '%s' (%s)" % (type, w.userdescription)
+            print("Added a widget of type '%s' (%s)" % (type, w.userdescription))
 
         return w.name
 
@@ -183,13 +185,13 @@ class CommandInterface(qt4.QObject):
         if ctype == 'colormap':
             self.document.validateProcessColormap(val)
         else:
-            if not isinstance(val, basestring):
-                raise RuntimeError, 'Value should be string'
+            if not isinstance(val, cbasestr):
+                raise RuntimeError('Value should be string')
 
         if mode not in ('appendalways', 'append', 'replace'):
-            raise RuntimeError, 'Invalid mode'
+            raise RuntimeError('Invalid mode')
         if ctype not in ('constant', 'import', 'function', 'colormap'):
-            raise RuntimeError, 'Invalid type'
+            raise RuntimeError('Invalid type')
 
         vals = list( self.document.customs )
         item = [ctype, name, val]
@@ -214,7 +216,7 @@ class CommandInterface(qt4.QObject):
 
     def AddImportPath(self, directory):
         """Add directory to import file path."""
-        assert isinstance(directory, basestring)
+        assert isinstance(directory, cbasestr)
         self.importpath.append(directory)
 
     def CloneWidget(self, widget, newparent, newname=None):
@@ -254,8 +256,8 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print ('Constructed histogram of "%s", creating datasets'
-                   ' "%s" and "%s"') % (inexpr, outbinsds, outvalsds)
+            print(('Constructed histogram of "%s", creating datasets'
+                   ' "%s" and "%s"') % (inexpr, outbinsds, outvalsds))
 
     def DatasetPlugin(self, pluginname, fields, datasetnames={}):
         """Use a dataset plugin.
@@ -272,7 +274,7 @@ class CommandInterface(qt4.QObject):
                 plugin = pkls()
                 break
         if plugin is None:
-            raise RuntimeError, "Cannot find dataset plugin '%s'" % pluginname
+            raise RuntimeError("Cannot find dataset plugin '%s'" % pluginname)
 
         # do the work
         op = operations.OperationDatasetPlugin(plugin, fields,
@@ -280,8 +282,8 @@ class CommandInterface(qt4.QObject):
         outdatasets = self.document.applyOperation(op)
 
         if self.verbose:
-            print "Used dataset plugin %s to make datasets %s" % (
-                pluginname, ', '.join(outdatasets))
+            print("Used dataset plugin %s to make datasets %s" % (
+                pluginname, ', '.join(outdatasets)))
 
     def Remove(self, name):
         """Remove a widget from the dataset."""
@@ -289,7 +291,7 @@ class CommandInterface(qt4.QObject):
         op = operations.OperationWidgetDelete(w)
         self.document.applyOperation(op)
         if self.verbose:
-            print "Removed widget '%s'" % name
+            print("Removed widget '%s'" % name)
 
     def RemoveCustom(self, name):
         """Removes a custom-defined constant, function or import."""
@@ -299,7 +301,7 @@ class CommandInterface(qt4.QObject):
                 del vals[i]
                 break
         else:
-            raise ValueError, 'Custom variable not defined'
+            raise ValueError('Custom variable not defined')
         op = operations.OperationSetCustom(vals)
         self.document.applyOperation(op)
 
@@ -313,7 +315,7 @@ class CommandInterface(qt4.QObject):
                                                    where)
 
         if self.verbose:
-            print "Changed to widget '%s'" % self.currentwidget.path
+            print("Changed to widget '%s'" % self.currentwidget.path)
 
     def List(self, where='.'):
         """List the contents of a widget, by default the current widget."""
@@ -322,12 +324,12 @@ class CommandInterface(qt4.QObject):
         children = widget.childnames
 
         if len(children) == 0:
-            print '%30s' % 'No children found'
+            print('%30s' % 'No children found')
         else:
             # output format name, type
             for name in children:
                 w = widget.getChild(name)
-                print '%10s %10s %30s' % (name, w.typename, w.userdescription)
+                print('%10s %10s %30s' % (name, w.typename, w.userdescription))
 
     def Get(self, var):
         """Get the value of a setting."""
@@ -341,9 +343,7 @@ class CommandInterface(qt4.QObject):
 
     def GetDatasets(self):
         """Return a list of names of datasets."""
-        ds = self.document.data.keys()
-        ds.sort()
-        return ds
+        return sorted(ckeys(self.document.data))
 
     def ResolveReference(self, setn):
         """If the setting is set to a reference, follow the chain of
@@ -372,8 +372,8 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print ( "Set setting '%s' to %s" %
-                    (var, repr(pref.get())) )
+            print(( "Set setting '%s' to %s" %
+                    (var, repr(pref.get())) ))
 
     def SetToReference(self, var, val):
         """Set setting to a reference value."""
@@ -383,8 +383,8 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print ( "Set setting '%s' to %s" %
-                    (var, repr(pref.get())) )
+            print(( "Set setting '%s' to %s" %
+                    (var, repr(pref.get())) ))
 
     def SetData(self, name, val, symerr=None, negerr=None, poserr=None):
         """Set dataset with name with values (and optionally errors)."""
@@ -394,11 +394,11 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
  
         if self.verbose:
-            print "Set dataset '%s':" % name
-            print " Values = %s" % str( data.data )
-            print " Symmetric errors = %s" % str( data.serr )
-            print " Negative errors = %s" % str( data.nerr )
-            print " Positive errors = %s" % str( data.perr )
+            print("Set dataset '%s':" % name)
+            print(" Values = %s" % str( data.data ))
+            print(" Symmetric errors = %s" % str( data.serr ))
+            print(" Negative errors = %s" % str( data.nerr ))
+            print(" Positive errors = %s" % str( data.perr ))
 
     def SetDataDateTime(self, name, vals):
         """Set datetime dataset to be values given.
@@ -410,8 +410,8 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Set dataset '%s':" % name
-            print " Values = %s" % str(ds.data)
+            print("Set dataset '%s':" % name)
+            print(" Values = %s" % str(ds.data))
 
     def SetDataExpression(self, name, val, symerr=None, negerr=None, poserr=None,
                           linked=False, parametric=None):
@@ -438,14 +438,14 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
         
         if self.verbose:
-            print "Set dataset '%s' based on expression:" % name
-            print " Values = %s" % str( data.data )
-            print " Symmetric errors = %s" % str( data.serr )
-            print " Negative errors = %s" % str( data.nerr )
-            print " Positive errors = %s" % str( data.perr )
+            print("Set dataset '%s' based on expression:" % name)
+            print(" Values = %s" % str( data.data ))
+            print(" Symmetric errors = %s" % str( data.serr ))
+            print(" Negative errors = %s" % str( data.nerr ))
+            print(" Positive errors = %s" % str( data.perr ))
             if parametric:
-                print " Where t goes form %g:%g in %i steps" % parametric
-            print " linked to expression = %s" % repr(linked)
+                print(" Where t goes form %g:%g in %i steps" % parametric)
+            print(" linked to expression = %s" % repr(linked))
 
     def SetDataRange(self, name, numsteps, val, symerr=None, negerr=None,
                      poserr=None, linked=False):
@@ -463,12 +463,12 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
         
         if self.verbose:
-            print "Set dataset '%s' based on range:" % name
-            print " Number of steps = %i" % numsteps
-            print " Range of data = %s" % repr(val)
-            print " Range of symmetric error = %s" % repr(symerr)
-            print " Range of positive error = %s" % repr(poserr)
-            print " Range of negative error = %s" % repr(negerr)
+            print("Set dataset '%s' based on range:" % name)
+            print(" Number of steps = %i" % numsteps)
+            print(" Range of data = %s" % repr(val))
+            print(" Range of symmetric error = %s" % repr(symerr))
+            print(" Range of positive error = %s" % repr(poserr))
+            print(" Range of negative error = %s" % repr(negerr))
 
     def SetData2DExpression(self, name, expr, linked=False):
         """Create a 2D dataset based on expressions
@@ -482,11 +482,11 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
 
         if self.verbose:
-            print "Set 2D dataset '%s' based on expressions" % name
-            print " expression = %s" % repr(expr)
-            print " linked to expression = %s" % repr(linked)
-            print " Made a dataset (%i x %i)" % (data.data.shape[0],
-                                                 data.data.shape[1])
+            print("Set 2D dataset '%s' based on expressions" % name)
+            print(" expression = %s" % repr(expr))
+            print(" linked to expression = %s" % repr(linked))
+            print(" Made a dataset (%i x %i)" % (data.data.shape[0],
+                                                 data.data.shape[1]))
 
     def SetData2DExpressionXYZ(self, name, xexpr, yexpr, zexpr, linked=False):
         """Create a 2D dataset based on expressions in x, y and z
@@ -502,13 +502,13 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
 
         if self.verbose:
-            print "Set 2D dataset '%s' based on expressions" % name
-            print " X expression = %s" % repr(xexpr)
-            print " Y expression = %s" % repr(yexpr)
-            print " Z expression = %s" % repr(zexpr)
-            print " linked to expression = %s" % repr(linked)
-            print " Made a dataset (%i x %i)" % (data.data.shape[0],
-                                                 data.data.shape[1])
+            print("Set 2D dataset '%s' based on expressions" % name)
+            print(" X expression = %s" % repr(xexpr))
+            print(" Y expression = %s" % repr(yexpr))
+            print(" Z expression = %s" % repr(zexpr))
+            print(" linked to expression = %s" % repr(linked))
+            print(" Made a dataset (%i x %i)" % (data.data.shape[0],
+                                                 data.data.shape[1]))
 
     def SetData2DXYFunc(self, name, xstep, ystep, expr, linked=False):
         """Create a 2D dataset based on expressions of a range of x and y
@@ -524,13 +524,13 @@ class CommandInterface(qt4.QObject):
         data = self.document.applyOperation(op)
 
         if self.verbose:
-            print "Set 2D dataset '%s' based on function of x and y" % name
-            print " X steps = %s" % repr(xstep)
-            print " Y steps = %s" % repr(ystep)
-            print " Expression = %s" % repr(expr)
-            print " linked to expression = %s" % repr(linked)
-            print " Made a dataset (%i x %i)" % (data.data.shape[0],
-                                                 data.data.shape[1])
+            print("Set 2D dataset '%s' based on function of x and y" % name)
+            print(" X steps = %s" % repr(xstep))
+            print(" Y steps = %s" % repr(ystep))
+            print(" Expression = %s" % repr(expr))
+            print(" linked to expression = %s" % repr(linked))
+            print(" Made a dataset (%i x %i)" % (data.data.shape[0],
+                                                 data.data.shape[1]))
 
     def SetData2D(self, name, data, xrange=None, yrange=None):
         """Create a 2D dataset."""
@@ -540,7 +540,7 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Set 2d dataset '%s'" % name
+            print("Set 2d dataset '%s'" % name)
 
     def SetDataText(self, name, val):
         """Create a text dataset."""
@@ -550,8 +550,8 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Set text dataset '%s'" % name
-            print " Values = %s" % unicode(data.data)
+            print("Set text dataset '%s'" % name)
+            print(" Values = %s" % cstr(data.data))
 
     def GetData(self, name):
         """Return the data with the name.
@@ -629,9 +629,9 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Imported datasets %s" % (' '.join(op.outdatasets),)
-            for name, num in op.outinvalids.iteritems():
-                print "%i errors encountered reading dataset %s" % (num, name)
+            print("Imported datasets %s" % (' '.join(op.outdatasets),))
+            for name, num in citems(op.outinvalids):
+                print("%i errors encountered reading dataset %s" % (num, name))
 
         return (op.outdatasets, op.outinvalids)
 
@@ -650,7 +650,7 @@ class CommandInterface(qt4.QObject):
 
         """
         
-        if type(datasetnames) in (str, unicode):
+        if isinstance(datasetnames, cbasestr):
             datasetnames = [datasetnames]
 
         params = importparams.ImportParams2D(
@@ -661,7 +661,7 @@ class CommandInterface(qt4.QObject):
         op = operations.OperationDataImport2D(params)
         self.document.applyOperation(op)
         if self.verbose:
-            print "Imported datasets %s" % (', '.join(datasetnames))
+            print("Imported datasets %s" % (', '.join(datasetnames)))
 
     def ImportFile2D(self, filename, datasetnames, xrange=None, yrange=None,
                      invertrows=None, invertcols=None, transpose=None,
@@ -688,7 +688,7 @@ class CommandInterface(qt4.QObject):
         # look up filename on path
         realfilename = self.findFileOnImportPath(filename)
 
-        if type(datasetnames) in (str, unicode):
+        if isinstance(datasetnames, cbasestr):
             datasetnames = [datasetnames]
 
         params = importparams.ImportParams2D(
@@ -701,7 +701,7 @@ class CommandInterface(qt4.QObject):
         op = operations.OperationDataImport2D(params)
         self.document.applyOperation(op)
         if self.verbose:
-            print "Imported datasets %s" % (', '.join(datasetnames))
+            print("Imported datasets %s" % (', '.join(datasetnames)))
 
     def ImportFile(self, filename, descriptor, useblocks=False, linked=False,
                    prefix='', suffix='', ignoretext=False, encoding='utf_8'):
@@ -734,9 +734,9 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Imported datasets %s" % (' '.join(op.outdatasets),)
-            for name, num in op.outinvalids.iteritems():
-                print "%i errors encountered reading dataset %s" % (num, name)
+            print("Imported datasets %s" % (' '.join(op.outdatasets),))
+            for name, num in citems(op.outinvalids):
+                print("%i errors encountered reading dataset %s" % (num, name))
 
         return (op.outdatasets, op.outinvalids)
 
@@ -797,7 +797,7 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Imported datasets %s" % (' '.join(op.outdatasets),)
+            print("Imported datasets %s" % (' '.join(op.outdatasets),))
 
         return op.outdatasets
 
@@ -966,7 +966,7 @@ class CommandInterface(qt4.QObject):
         if hasattr(item, 'isWidget') and item.isWidget():
             return item.typename
         else:
-            raise ValueError, "Path '%s' is not a widget" % path
+            raise ValueError("Path '%s' is not a widget" % path)
 
     def SettingType(self, path):
         """Get the type of setting (a string) for the path given.
@@ -983,5 +983,5 @@ class CommandInterface(qt4.QObject):
         self.document.applyOperation(op)
 
         if self.verbose:
-            print "Applied tag %s to datasets %s" % (
-                tag, ' '.join(datasets))
+            print("Applied tag %s to datasets %s" % (
+                tag, ' '.join(datasets)))
