@@ -19,19 +19,19 @@
 
 """Classes to represent datasets."""
 
+from __future__ import division
 import re
-from itertools import izip
 
 import numpy as N
 
-import veusz.qtall as qt4
-import veusz.utils as utils
-import veusz.setting as setting
+from ..compat import czip, crange, citems, cbasestr, cstr, crepr
+from .. import qtall as qt4
+from .. import utils
+from .. import setting
 
 def _(text, disambiguation=None, context="Datasets"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 def convertNumpy(a, dims=1):
     """Convert to a numpy double if possible.
@@ -50,7 +50,7 @@ def convertNumpy(a, dims=1):
         a = N.array(a, dtype=N.float64)
 
     if len(a.shape) != dims:
-        raise ValueError, "Only %i-dimensional arrays or lists allowed" % dims
+        raise ValueError("Only %i-dimensional arrays or lists allowed" % dims)
     return a
 
 def convertNumpyAbs(a):
@@ -178,7 +178,7 @@ class DatasetBase(object):
 
     def name(self):
         """Get dataset name."""
-        for name, ds in self.document.data.iteritems():
+        for name, ds in citems(self.document.data):
             if ds == self:
                 return name
         raise ValueError('Could not find self in document.data')
@@ -200,15 +200,15 @@ class DatasetBase(object):
         """Return a value cast to this dataset data type.
         We assume here it is a float, so override if not
         """
-        if isinstance(val, basestring) or isinstance(val, qt4.QString):
+        if isinstance(val, cbasestr):
             val, ok = setting.uilocale.toDouble(val)
             if ok: return val
-            raise ValueError, "Invalid floating point number"
+            raise ValueError("Invalid floating point number")
         return float(val)
 
-    def uiDataItemToQVariant(self, val):
-        """Return val converted to QVariant."""
-        return qt4.QVariant(float(val))
+    def uiDataItemToData(self, val):
+        """Return val converted to data."""
+        return float(val)
     
     def _getItemHelper(self, key):
         """Help get arguments to constructor."""
@@ -265,6 +265,10 @@ class DatasetBase(object):
         """Return dataset as text (for use by user)."""
         return ''
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return True
+
 class Dataset2D(DatasetBase):
     '''Represents a two-dimensional dataset.'''
 
@@ -322,7 +326,7 @@ class Dataset2D(DatasetBase):
         if self.linked is not None:
             return
 
-        fileobj.write("ImportString2D(%s, '''\n" % repr(name))
+        fileobj.write("ImportString2D(%s, '''\n" % crepr(name))
         fileobj.write("xrange %e %e\n" % tuple(self.xrange))
         fileobj.write("yrange %e %e\n" % tuple(self.yrange))
         fileobj.write(self.datasetAsText(fmt='%e', join=' '))
@@ -512,7 +516,7 @@ class Dataset(DatasetBase):
         if thetype in self.columns:
             setattr(self, thetype, vals)
         else:
-            raise ValueError, 'thetype does not contain an allowed value'
+            raise ValueError('thetype does not contain an allowed value')
 
         # just a check...
         s = self.data.shape
@@ -539,7 +543,7 @@ class Dataset(DatasetBase):
         if self.nerr is not None:
             descriptor += ',-'
 
-        fileobj.write( "ImportString(%s,'''\n" % repr(descriptor) )
+        fileobj.write( "ImportString(%s,'''\n" % crepr(descriptor) )
         fileobj.write( self.datasetAsText(fmt='%e', join=' ') )
         fileobj.write( "''')\n" )
 
@@ -557,7 +561,7 @@ class Dataset(DatasetBase):
 
         # do the conversion
         lines = []
-        for line in izip(*cols):
+        for line in czip(*cols):
             lines.append( format % line )
         return ''.join(lines)
 
@@ -618,8 +622,8 @@ class DatasetDateTime(Dataset):
 
     def uiConvertToDataItem(self, val):
         """Return a value cast to this dataset data type."""
-        if isinstance(val, basestring) or isinstance(val, qt4.QString):
-            v = utils.dateStringToDate( unicode(val) )
+        if isinstance(val, cbasestr):
+            v = utils.dateStringToDate( cstr(val) )
             if not N.isfinite(v):
                 try:
                     v = float(val)
@@ -629,9 +633,9 @@ class DatasetDateTime(Dataset):
         else:
             return N.nan
 
-    def uiDataItemToQVariant(self, val):
-        """Return val converted to QVariant."""
-        return qt4.QVariant(utils.dateFloatToString(val))
+    def uiDataItemToData(self, val):
+        """Return val converted to data."""
+        return utils.dateFloatToString(val)
 
     def saveToFile(self, fileobj, name):
         '''Save data to file.
@@ -642,7 +646,7 @@ class DatasetDateTime(Dataset):
             return
 
         descriptor = datasetNameToDescriptorName(name) + '(date)'
-        fileobj.write( "ImportString(%s,'''\n" % repr(descriptor) )
+        fileobj.write( "ImportString(%s,'''\n" % crepr(descriptor) )
         fileobj.write( self.datasetAsText() )
         fileobj.write( "''')\n" )
 
@@ -686,17 +690,17 @@ class DatasetText(DatasetBase):
         if type == 'data':
             self.data = list(vals)
         else:
-            raise ValueError, 'type does not contain an allowed value'
+            raise ValueError('type does not contain an allowed value')
 
         self.document.modifiedData(self)
     
     def uiConvertToDataItem(self, val):
         """Return a value cast to this dataset data type."""
-        return unicode(val)
+        return cstr(val)
 
-    def uiDataItemToQVariant(self, val):
-        """Return val converted to QVariant."""
-        return qt4.QVariant(unicode(val))
+    def uiDataItemToData(self, val):
+        """Return val converted to data."""
+        return cstr(val)
 
     def saveToFile(self, fileobj, name):
         '''Save data to file.
@@ -707,7 +711,7 @@ class DatasetText(DatasetBase):
             return
 
         descriptor = datasetNameToDescriptorName(name) + '(text)'
-        fileobj.write( "ImportString(%s,r'''\n" % repr(descriptor) )
+        fileobj.write( "ImportString(%s, ur'''\n" % crepr(descriptor) )
         for line in self.data:
             # need to "escape" ''' marks in text
             r = repr(line).replace("'''", "''' \"'''\" r'''") + '\n'
@@ -783,7 +787,7 @@ def _substituteDatasets(datasets, expression, thispart):
 
         if bit in datasets:
             # replace name with a function to call
-            bits[i] = "_DS_(%s, %s)" % (repr(bit), repr(part))
+            bits[i] = "_DS_(%s, %s)" % (crepr(bit), crepr(part))
             dslist.append(bit)
 
     return ''.join(bits), dslist
@@ -827,7 +831,7 @@ def _returnNumericDataset(doc, vals, dimensions, subdatasets):
             minlen = len(vals[0])
             if len(vals) in (2,3):
                 # expand/convert error bars
-                for i in xrange(1, len(vals)):
+                for i in crange(1, len(vals)):
                     if N.isscalar(vals[i]):
                         # convert to scalar
                         vals[i] = N.zeros(minlen) + vals[i]
@@ -839,7 +843,7 @@ def _returnNumericDataset(doc, vals, dimensions, subdatasets):
                     minlen = min(minlen, len(vals[i]))
 
                 # chop to minimum length
-                for i in xrange(len(vals)):
+                for i in crange(len(vals)):
                     vals[i] = vals[i][:minlen]
             vals = N.array(vals, dtype=N.float64)
             err = None
@@ -878,7 +882,6 @@ def _returnNumericDataset(doc, vals, dimensions, subdatasets):
 
     raise DatasetExpressionException(err)
 
-_safeexpr = set()
 def evalDatasetExpression(doc, origexpr, datatype='numeric',
                           dimensions=1, part='data'):
     """Evaluate expression and return an appropriate Dataset.
@@ -908,13 +911,9 @@ def evalDatasetExpression(doc, origexpr, datatype='numeric',
     # replace dataset names by calls to _DS_(name,part)
     expr, subdatasets = _substituteDatasets(doc.data, origexpr, part)
 
-    # for speed, track safe expressions
-    if expr not in _safeexpr:
-        if ( not setting.transient_settings['unsafe_mode'] and
-             utils.checkCode(expr, securityonly=True) ):
-            doc.log("Unsafe expression: '%s'\n" % expr)
-            return None
-        _safeexpr.add(expr)
+    comp = doc.compileCheckedExpression(expr, origexpr=origexpr)
+    if comp is None:
+        return
 
     # set up environment for evaluation
     env = doc.eval_context.copy()
@@ -924,9 +923,9 @@ def evalDatasetExpression(doc, origexpr, datatype='numeric',
 
     # do evaluation
     try:
-        evalout = eval(expr, env)
-    except Exception, ex:
-        doc.log("Error evaluating '%s': '%s'" % (origexpr, unicode(ex)))
+        evalout = eval(comp, env)
+    except Exception as ex:
+        doc.log("Error evaluating '%s': '%s'" % (origexpr, cstr(ex)))
         return None
 
     # return correct dataset for data type
@@ -934,11 +933,11 @@ def evalDatasetExpression(doc, origexpr, datatype='numeric',
         if datatype == 'numeric':
             return _returnNumericDataset(doc, evalout, dimensions, subdatasets)
         elif datatype == 'text':
-            return DatasetText([unicode(x) for x in evalout])
+            return DatasetText([cstr(x) for x in evalout])
         else:
             raise RuntimeError('Invalid data type')
-    except DatasetExpressionException, ex:
-        doc.log(_("Error evaluating '%s': %s\n") % (origexpr, unicode(ex)))
+    except DatasetExpressionException as ex:
+        doc.log(_("Error evaluating '%s': %s\n") % (origexpr, cstr(ex)))
 
     return None
 
@@ -964,11 +963,13 @@ class DatasetExpression(Dataset):
         self.expr['perr'] = perr
         self.parametric = parametric
 
-        self.cachedexpr = {}
-
         self.docchangeset = -1
         self.evaluated = {}
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     def evaluateDataset(self, dsname, dspart):
         """Return the dataset given.
         
@@ -976,20 +977,18 @@ class DatasetExpression(Dataset):
         dspart is the part to get (e.g. data, serr)
         """
         return _evaluateDataset(self.document.data, dsname, dspart)
-                    
-    def _evaluatePart(self, expr, part):
-        """Evaluate expression expr for part part."""
-        # replace dataset names with calls
-        expr = _substituteDatasets(self.document.data, expr, part)[0]
 
-        # check expression for nasties if it has changed
-        if self.cachedexpr.get(part) != expr:
-            if ( not setting.transient_settings['unsafe_mode'] and
-                 utils.checkCode(expr, securityonly=True) ):
-                raise DatasetExpressionException(
-                    _("Unsafe expression '%s' in %s part of dataset") % (
-                        self.expr[part], part))
-            self.cachedexpr[part] = expr
+    def _evaluatePart(self, expr, part):
+        """Evaluate expression expr for part part.
+
+        Returns True if succeeded
+        """
+        # replace dataset names with calls
+        newexpr = _substituteDatasets(self.document.data, expr, part)[0]
+
+        comp = self.document.compileCheckedExpression(newexpr, origexpr=expr)
+        if comp is None:
+            return False
 
         # set up environment to evaluate expressions in
         environment = self.document.eval_context.copy()
@@ -1009,15 +1008,16 @@ class DatasetExpression(Dataset):
 
         # actually evaluate the expression
         try:
-            result = eval(expr, environment)
+            result = eval(comp, environment)
             evalout = N.array(result, N.float64)
 
             if len(evalout.shape) > 1:
-                raise RuntimeError, "Number of dimensions is not 1"
-        except Exception, ex:
-            raise DatasetExpressionException(
-                "Error evaluating expression: %s\n"
-                "Error: %s" % (self.expr[part], unicode(ex)) )
+                raise RuntimeError("Number of dimensions is not 1")
+        except Exception as ex:
+            self.document.log(
+                _("Error evaluating expression: %s\n"
+                  "Error: %s") % (self.expr[part], cstr(ex)) )
+            return False
 
         # make evaluated error expression have same shape as data
         if part != 'data':
@@ -1036,11 +1036,14 @@ class DatasetExpression(Dataset):
                 evalout = N.resize(evalout, 1)
 
         self.evaluated[part] = evalout
+        return True
 
     def updateEvaluation(self):
         """Update evaluation of parts of dataset.
-        Throws DatasetExpressionException if error
+
+        Returns False if problem with any evaluation
         """
+        ok = True
         if self.docchangeset != self.document.changeset:
             # avoid infinite recursion!
             self.docchangeset = self.document.changeset
@@ -1053,15 +1056,15 @@ class DatasetExpression(Dataset):
             for part in self.columns:
                 expr = self.expr[part]
                 if expr is not None and expr.strip() != '':
-                    self._evaluatePart(expr, part)
+                    ok = ok and self._evaluatePart(expr, part)
+
+        return ok
 
     def _propValues(self, part):
         """Check whether expressions need reevaluating,
         and recalculate if necessary."""
-        try:
-            self.updateEvaluation()
-        except DatasetExpressionException, ex:
-            self.document.log(unicode(ex))
+
+        self.updateEvaluation()
 
         # catch case where error in setting data, need to return "real" data
         if self.evaluated['data'] is None:
@@ -1079,21 +1082,21 @@ class DatasetExpression(Dataset):
         '''Save data to file.
         '''
 
-        parts = [repr(name), repr(self.expr['data'])]
+        parts = [crepr(name), crepr(self.expr['data'])]
         if self.expr['serr']:
-            parts.append('symerr=%s' % repr(self.expr['serr']))
+            parts.append('symerr=%s' % crepr(self.expr['serr']))
         if self.expr['nerr']:
-            parts.append('negerr=%s' % repr(self.expr['nerr']))
+            parts.append('negerr=%s' % crepr(self.expr['nerr']))
         if self.expr['perr']:
-            parts.append('poserr=%s' % repr(self.expr['perr']))
+            parts.append('poserr=%s' % crepr(self.expr['perr']))
         if self.parametric is not None:
-            parts.append('parametric=%s' % repr(self.parametric))
+            parts.append('parametric=%s' % crepr(self.parametric))
 
         parts.append('linked=True')
 
         s = 'SetDataExpression(%s)\n' % ', '.join(parts)
         fileobj.write(s)
-        
+
     def __getitem__(self, key):
         """Return a dataset based on this dataset
 
@@ -1119,7 +1122,7 @@ class DatasetExpression(Dataset):
             text.append(_('Linked parametric dataset'))
         else:
             text.append(_('Linked expression dataset'))
-        for label, part in izip(self.column_descriptions,
+        for label, part in czip(self.column_descriptions,
                                 self.columns):
             if self.expr[part]:
                 text.append('%s: %s' % (label, self.expr[part]))
@@ -1162,6 +1165,10 @@ class DatasetRange(Dataset):
                 vals = None
             setattr(self, name, vals)
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     def __getitem__(self, key):
         """Return a dataset based on this dataset
 
@@ -1177,13 +1184,13 @@ class DatasetRange(Dataset):
     def saveToFile(self, fileobj, name):
         """Save dataset to file."""
 
-        parts = [repr(name), repr(self.numsteps), repr(self.range_data)]
+        parts = [crepr(name), crepr(self.numsteps), crepr(self.range_data)]
         if self.range_serr is not None:
-            parts.append('symerr=%s' % repr(self.range_serr))
+            parts.append('symerr=%s' % crepr(self.range_serr))
         if self.range_perr is not None:
-            parts.append('poserr=%s' % repr(self.range_perr))
+            parts.append('poserr=%s' % crepr(self.range_perr))
         if self.range_nerr is not None:
-            parts.append('negerr=%s' % repr(self.range_nerr))
+            parts.append('negerr=%s' % crepr(self.range_nerr))
         parts.append('linked=True')
 
         s = 'SetDataRange(%s)\n' % ', '.join(parts)
@@ -1195,7 +1202,7 @@ class DatasetRange(Dataset):
     def linkedInformation(self):
         """Return information about linking."""
         text = [_('Linked range dataset')]
-        for label, part in izip(self.column_descriptions,
+        for label, part in czip(self.column_descriptions,
                                 self.columns):
             val = getattr(self, 'range_%s' % part)
             if val:
@@ -1254,8 +1261,9 @@ class Dataset2DXYZExpression(Dataset2D):
         self.expry = expry
         self.exprz = exprz
 
-        # cache x y and z expressions
-        self.cachedexpr = {}
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
 
     def evaluateDataset(self, dsname, dspart):
         """Return the dataset given.
@@ -1270,6 +1278,8 @@ class Dataset2DXYZExpression(Dataset2D):
         # return cached data if document unchanged
         if self.document.changeset == self.lastchangeset:
             return self.cacheddata
+        self.lastchangeset = self.document.changeset
+        self.cacheddata = None
 
         evaluated = {}
 
@@ -1278,24 +1288,21 @@ class Dataset2DXYZExpression(Dataset2D):
 
         # evaluate the x, y and z expressions
         for name in ('exprx', 'expry', 'exprz'):
-            expr = _substituteDatasets(self.document.data, getattr(self, name),
+            origexpr = getattr(self, name)
+            expr = _substituteDatasets(self.document.data, origexpr,
                                        'data')[0]
 
-            # check expression if not checked before
-            if self.cachedexpr.get(name) != expr:
-                if ( not setting.transient_settings['unsafe_mode'] and
-                     utils.checkCode(expr, securityonly=True) ):
-                    raise DatasetExpressionException(
-                        "Unsafe expression '%s'" % (
-                            expr))
-                self.cachedexpr[name] = expr
+            comp = self.document.compileCheckedExpression(
+                expr, origexpr=origexpr)
+            if comp is None:
+                return None
 
             try:
-                evaluated[name] = eval(expr, environment)
-            except Exception, e:
-                raise DatasetExpressionException(
-                    "Error evaluating expression: %s\n"
-                    "Error: %s" % (expr, unicode(e)) )
+                evaluated[name] = eval(comp, environment)
+            except Exception as e:
+                self.document.log(_("Error evaluating expression: %s\n"
+                                    "Error: %s") % (expr, cstr(e)) )
+                return None
 
         minx, maxx, stepx, stepsx = getSpacing(evaluated['exprx'])
         miny, maxy, stepy, stepsy = getSpacing(evaluated['expry'])
@@ -1312,13 +1319,10 @@ class Dataset2DXYZExpression(Dataset2D):
         # this is ugly - is this really the way to do it?
         try:
             self.cacheddata.flat [ xpts + ypts*stepsx ] = evaluated['exprz']
-        except Exception, e:
-            raise DatasetExpressionException(
-                "Shape mismatch when constructing dataset\n"
-                "Error: %s" % unicode(e) )
-
-        # update changeset
-        self.lastchangeset = self.document.changeset
+        except Exception as e:
+            self.document.log(_("Shape mismatch when constructing dataset\n"
+                                "Error: %s") % cstr(e) )
+            return None
 
         return self.cacheddata
 
@@ -1326,28 +1330,26 @@ class Dataset2DXYZExpression(Dataset2D):
     def xrange(self):
         """Get x range of data as a tuple (min, max)."""
         return self.getDataRanges()[0]
-    
+
     @property
     def yrange(self):
         """Get y range of data as a tuple (min, max)."""
         return self.getDataRanges()[1]
-        
+
     def getDataRanges(self):
         """Get both ranges of axis."""
-        try:
-            self.evalDataset()
-            return (self._xrange, self._yrange)
-        except DatasetExpressionException:
+        ds = self.evalDataset()
+        if ds is None:
             return ( (0., 1.), (0., 1.) )
-        
+        return (self._xrange, self._yrange)
+
     @property
     def data(self):
         """Get data, or none if error."""
-        try:
-            return self.evalDataset()
-        except DatasetExpressionException, ex:
-            self.document.log(unicode(ex))
+        ds = self.evalDataset()
+        if ds is None:
             return N.array( [[]] )
+        return ds
 
     def description(self, showlinked=True):
         # FIXME: dataeditdialog descriptions should be taken from here somewhere
@@ -1361,7 +1363,7 @@ class Dataset2DXYZExpression(Dataset2D):
         '''
 
         s = 'SetData2DExpressionXYZ(%s, %s, %s, %s, linked=True)\n' % (
-            repr(name), repr(self.exprx), repr(self.expry), repr(self.exprz) )
+            crepr(name), crepr(self.exprx), crepr(self.expry), crepr(self.exprz) )
         fileobj.write(s)
 
     def canUnlink(self):
@@ -1386,8 +1388,9 @@ class Dataset2DExpression(Dataset2D):
         self.expr = expr
         self.lastchangeset = -1
 
-        if utils.checkCode(expr, securityonly=True) is not None:
-            raise DatasetExpressionException("Unsafe expression '%s'" % expr)
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
         
     @property
     def data(self):
@@ -1420,7 +1423,7 @@ class Dataset2DExpression(Dataset2D):
     def saveToFile(self, fileobj, name):
         '''Save expression to file.'''
         s = 'SetData2DExpression(%s, %s, linked=True)\n' % (
-            repr(name), repr(self.expr) )
+            crepr(name), crepr(self.expr) )
         fileobj.write(s)
 
     def canUnlink(self):
@@ -1452,23 +1455,25 @@ class Dataset2DXYFunc(Dataset2D):
         self.ystep = ystep
         self.expr = expr
 
-        if utils.checkCode(expr, securityonly=True) is not None:
-            raise DatasetExpressionException(_("Unsafe expression '%s'") % expr)
-        
         self.xrange = (self.xstep[0] - self.xstep[2]*0.5,
                        self.xstep[1] + self.xstep[2]*0.5)
         self.yrange = (self.ystep[0] - self.ystep[2]*0.5,
                        self.ystep[1] + self.ystep[2]*0.5)
 
+        self.cacheddata = None
         self.lastchangeset = -1
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     @property
     def data(self):
         """Return data, or empty array if error."""
         try:
             return self.evalDataset()
-        except DatasetExpressionException, ex:
-            self.document.log(unicode(ex))
+        except DatasetExpressionException as ex:
+            self.document.log(cstr(ex))
             return N.array([[]])
 
     def evalDataset(self):
@@ -1491,7 +1496,7 @@ class Dataset2DXYFunc(Dataset2D):
         env['y'] = ystep
         try:
             data = eval(self.expr, env)
-        except Exception, e:
+        except Exception as e:
             raise DatasetExpressionException(
                 _("Error evaluating expression: %s\n"
                   "Error: %s") % (self.expr, str(e)) )
@@ -1507,7 +1512,7 @@ class Dataset2DXYFunc(Dataset2D):
         '''Save expressions to file.
         '''
         s = 'SetData2DXYFunc(%s, %s, %s, %s, linked=True)\n' % (
-            repr(name), repr(self.xstep), repr(self.ystep), repr(self.expr) )
+            crepr(name), crepr(self.xstep), crepr(self.ystep), crepr(self.expr) )
         fileobj.write(s)
 
     def canUnlink(self):
@@ -1534,8 +1539,8 @@ class _DatasetPlugin(object):
         """Return information about how this dataset was created."""
 
         fields = []
-        for name, val in self.pluginmanager.fields.iteritems():
-            fields.append('%s: %s' % (unicode(name), unicode(val)))
+        for name, val in citems(self.pluginmanager.fields):
+            fields.append('%s: %s' % (cstr(name), cstr(val)))
 
         try:
             shape = [str(x) for x in self.data.shape]
@@ -1584,6 +1589,10 @@ class Dataset1DPlugin(_DatasetPlugin, Dataset):
         _DatasetPlugin.__init__(self, manager, ds)
         Dataset.__init__(self, data=[])
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     def userSize(self):
         """Size of dataset."""
         return str( self.data.shape[0] )
@@ -1613,6 +1622,10 @@ class Dataset2DPlugin(_DatasetPlugin, Dataset2D):
         _DatasetPlugin.__init__(self, manager, ds)
         Dataset2D.__init__(self, [[]])
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     def __getitem__(self, key):
         return Dataset2D(self.data[key], xrange=self.xrange, yrange=self.yrange)
         
@@ -1630,6 +1643,10 @@ class DatasetTextPlugin(_DatasetPlugin, DatasetText):
         _DatasetPlugin.__init__(self, manager, ds)
         DatasetText.__init__(self, [])
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     def __getitem__(self, key):
         return DatasetText(self.data[key])
 
@@ -1643,6 +1660,10 @@ class DatasetDateTimePlugin(_DatasetPlugin, DatasetDateTime):
         _DatasetPlugin.__init__(self, manager, ds)
         DatasetDateTime.__init__(self, [])
 
+    def editable(self):
+        """Is the dataset editable?"""
+        return False
+
     def __getitem__(self, key):
         return DatasetDateTime(self.data[key])
 
