@@ -21,6 +21,7 @@
 
 """A class to represent Veusz documents, with dataset classes."""
 
+from __future__ import division
 import os.path
 import re
 import traceback
@@ -29,19 +30,19 @@ from collections import defaultdict
 
 import numpy as N
 
-import veusz.qtall as qt4
+from ..compat import crange, citems, cvalues, cstr, cexec
+from .. import qtall as qt4
 
-import widgetfactory
-import datasets
-import painthelper
+from . import widgetfactory
+from . import datasets
+from . import painthelper
 
-import veusz.utils as utils
-import veusz.setting as setting
+from .. import utils
+from .. import setting
 
 def _(text, disambiguation=None, context="Document"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # python identifier
 identifier_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_]*$')
@@ -81,6 +82,9 @@ class Document( qt4.QObject ):
 
     pluginsloaded = False
 
+    # this is emitted when the document is modified
+    signalModified = qt4.pyqtSignal(int)
+
     def __init__(self):
         """Initialise the document."""
         qt4.QObject.__init__( self )
@@ -122,6 +126,11 @@ class Document( qt4.QObject ):
         # copy default colormaps
         self.colormaps = dict(utils.defaultcolormaps)
 
+        # copies of validated compiled expressions
+        self.exprcompiled = {}
+        self.exprfailed = set()
+        self.exprfailedchangeset = -1
+
     def wipe(self):
         """Wipe out any stored data."""
         self.data = {}
@@ -299,7 +308,7 @@ class Document( qt4.QObject ):
 
     def modifiedData(self, dataset):
         """The named dataset was modified"""
-        for name, ds in self.data.iteritems():
+        for name, ds in citems(self.data):
             if ds is dataset:
                 self.datachangesets[name] += 1
                 self.datachangeset += 1
@@ -310,7 +319,7 @@ class Document( qt4.QObject ):
         if filenames is a set, only get the objects with filenames given
         """
         links = set()
-        for ds in self.data.itervalues():
+        for ds in cvalues(self.data):
             if ds.linked and (filenames is None or
                               ds.linked.filename in filenames):
                 links.add(ds.linked)
@@ -343,10 +352,10 @@ class Document( qt4.QObject ):
 
     def datasetName(self, dataset):
         """Find name for given dataset, raising ValueError if missing."""
-        for name, ds in self.data.iteritems():
+        for name, ds in citems(self.data):
             if ds is dataset:
                 return name
-        raise ValueError, "Cannot find dataset"
+        raise ValueError("Cannot find dataset")
 
     def deleteDataset(self, name):
         """Remove the selected dataset."""
@@ -382,7 +391,7 @@ class Document( qt4.QObject ):
         self.changeset += 1
 
         if len(self.suspendupdates) == 0:
-            self.emit( qt4.SIGNAL("sigModified"), ismodified )
+            self.signalModified.emit(ismodified)
 
     def isModified(self):
         """Return whether modified flag set."""
@@ -396,7 +405,7 @@ class Document( qt4.QObject ):
 
         for plugin in pluginlist:
             try:
-                execfile(plugin, dict())
+                cexec(compile(open(plugin).read(), plugin, 'exec'), dict())
             except Exception:
                 err = _('Error loading plugin %s\n\n%s') % (
                     plugin, traceback.format_exc())
@@ -440,9 +449,9 @@ class Document( qt4.QObject ):
     def datasetTags(self):
         """Get list of all tags in datasets."""
         tags = set()
-        for dataset in self.data.itervalues():
+        for dataset in cvalues(self.data):
             tags.update(dataset.tags)
-        return list(sorted(tags))
+        return sorted(tags)
 
     def _writeFileHeader(self, fileobj, type):
         """Write a header to a saved file of type."""
@@ -463,12 +472,12 @@ class Document( qt4.QObject ):
 
         # get a list of all tags and which datasets have them
         bytag = defaultdict(list)
-        for name, dataset in sorted(self.data.iteritems()):
+        for name, dataset in sorted(citems(self.data)):
             for t in dataset.tags:
                 bytag[t].append(name)
 
         # write out tags
-        for tag, val in sorted(bytag.iteritems()):
+        for tag, val in sorted(citems(bytag)):
             fileobj.write('TagDatasets(%s, %s)\n' %
                           (repr(tag), repr(val)))
 
@@ -495,12 +504,12 @@ class Document( qt4.QObject ):
         # save those datasets which are linked
         # we do this first in case the datasets are overridden below
         savedlinks = {}
-        for name, dataset in sorted(self.data.items()):
+        for name, dataset in sorted(citems(self.data)):
             dataset.saveLinksToSavedDoc(fileobj, savedlinks,
                                         relpath=reldirname)
 
         # save the remaining datasets
-        for name, dataset in sorted(self.data.items()):
+        for name, dataset in sorted(citems(self.data)):
             dataset.saveToFile(fileobj, name)
 
         # save tags of datasets
@@ -563,7 +572,7 @@ class Document( qt4.QObject ):
             if p == '..':
                 p = obj.parent
                 if p is None:
-                    raise ValueError, "Base graph has no parent"
+                    raise ValueError("Base graph has no parent")
                 obj = p
             elif p == '.' or len(p) == 0:
                 pass
@@ -576,15 +585,15 @@ class Document( qt4.QObject ):
                         if p in obj.settings:
                             obj = obj.settings[p]
                         else:
-                            raise ValueError, "Widget has no child %s" % p
+                            raise ValueError("Widget has no child %s" % p)
                 else:
                     if isinstance(obj, setting.Settings):
                         try:
                             obj = obj.get(p)
                         except KeyError:
-                            raise ValueError, "Settings has no child %s" % p
+                            raise ValueError("Settings has no child %s" % p)
                     else:
-                        raise ValueError, "Item has no children"
+                        raise ValueError("Item has no children")
 
         # return widget
         return obj
@@ -611,7 +620,7 @@ class Document( qt4.QObject ):
                 # relative to parent object
                 p = obj.parent
                 if p is None:
-                    raise ValueError, "Base graph has no parent"
+                    raise ValueError("Base graph has no parent")
                 obj = p
             elif p == '.' or len(p) == 0:
                 # relative to here
@@ -620,7 +629,7 @@ class Document( qt4.QObject ):
                 # child specified
                 obj = obj.getChild( p )
                 if obj is None:
-                    raise ValueError, "Child '%s' does not exist" % p
+                    raise ValueError("Child '%s' does not exist" % p)
 
         # return widget
         return obj
@@ -637,7 +646,7 @@ class Document( qt4.QObject ):
             return symbols
 
         # two-pass to ask user whether they want to import symbol
-        for thepass in xrange(2):
+        for thepass in crange(2):
             # remembered during session
             a = 'import_allowed'
             if a not in setting.transient_settings:
@@ -688,7 +697,7 @@ class Document( qt4.QObject ):
                 defn = 'from %s import %s' % (module,
                                               ', '.join(toimport))
                 try:
-                    exec defn in self.eval_context
+                    cexec(defn, self.eval_context)
                 except Exception:
                     self.log(_("Failed to import '%s' from "
                                "module '%s'") % (', '.join(toimport),
@@ -743,10 +752,10 @@ class Document( qt4.QObject ):
 
         try:
             cmap = self.validateProcessColormap(val)
-        except ValueError, e:
-            self.log( unicode(e) )
+        except ValueError as e:
+            self.log( cstr(e) )
         else:
-            self.colormaps[ unicode(name) ] = cmap
+            self.colormaps[ cstr(name) ] = cmap
 
     def _updateEvalContextFuncOrConst(self, ctype, name, val):
         """Update a function or constant in eval function context."""
@@ -766,16 +775,53 @@ class Document( qt4.QObject ):
             defn = 'lambda %s: %s' % (args, val)
 
         # evaluate, but we ignore any unsafe commands or exceptions
-        checked = utils.checkCode(defn)
-        if checked is not None:
-            self.log( _("Expression '%s' failed safe code test") %
-                      defn )
+        comp = self.compileCheckedExpression(defn)
+        if comp is None:
             return
         try:
-            self.eval_context[name] = eval(defn, self.eval_context)
-        except Exception, e:
+            self.eval_context[name] = eval(comp, self.eval_context)
+        except Exception as e:
             self.log( _("Error evaluating '%s': '%s'") %
-                      (name, unicode(e)) )
+                      (name, cstr(e)) )
+
+    def compileCheckedExpression(self, expr, origexpr=None):
+        """Compile expression and check for errors.
+
+        origexpr is an expression to show in error messages. This is
+        used if replacements have been done, etc.
+        """
+
+        try:
+            return self.exprcompiled[expr]
+        except KeyError:
+            pass
+
+        # track failed compilations, so we only print them once
+        if self.exprfailedchangeset != self.changeset:
+            self.exprfailedchangeset = self.changeset
+            self.exprfailed.clear()
+        elif expr in self.exprfailed:
+            return None
+
+        if origexpr is None:
+            origexpr = expr
+
+        try:
+            checked = utils.compileChecked(
+                expr,
+                ignoresecurity=setting.transient_settings['unsafe_mode'])
+        except utils.SafeEvalException as e:
+            self.log(
+                _("Unsafe expression '%s': %s") % (origexpr, cstr(e)))
+            self.exprfailed.add(expr)
+            return None
+        except Exception as e:
+            self.log(
+                _("Error in expression '%s': %s") % (origexpr, cstr(e)))
+            return None
+        else:
+            self.exprcompiled[expr] = checked
+            return checked
 
     def updateEvalContext(self):
         """To be called after custom constants or functions are changed.
@@ -786,7 +832,7 @@ class Document( qt4.QObject ):
 
         # add numpy things
         # we try to avoid various bits and pieces for safety
-        for name, val in N.__dict__.iteritems():
+        for name, val in citems(N.__dict__):
             if ( (callable(val) or type(val)==float) and
                  name not in __builtins__ and
                  name[:1] != '_' and name[-1:] != '_' ):
@@ -807,7 +853,7 @@ class Document( qt4.QObject ):
             elif ctype == 'colormap':
                 self._updateEvalContextColormap(name, val)
             else:
-                raise ValueError, 'Invalid custom type'
+                raise ValueError('Invalid custom type')
 
     def customDict(self):
         """Return a dictionary mapping custom names to (idx, type, value)."""
@@ -854,11 +900,11 @@ class Document( qt4.QObject ):
 
         elif datatype == 'text':
             try:
-                return datasets.DatasetText([unicode(x) for x in vals])
+                return datasets.DatasetText([cstr(x) for x in vals])
             except ValueError:
                 pass
 
-        raise RuntimError, 'Invalid array'
+        raise RuntimeError('Invalid array')
 
     def walkNodes(self, tocall, root=None,
                   nodetypes=('widget', 'setting', 'settings'),
@@ -890,7 +936,7 @@ class Document( qt4.QObject ):
                            nodetypes=nodetypes, _path=_path)
         elif root.nodetype == 'settings':
             # do the settings of the settings
-            for name, s in sorted(root.setdict.iteritems()):
+            for name, s in sorted(citems(root.setdict)):
                 self.walkNodes(tocall, root=s, nodetypes=nodetypes,
                                _path = _path + '/' + s.name)
         # elif root.nodetype == 'setting': pass
@@ -905,3 +951,17 @@ class Document( qt4.QObject ):
                 # ignore marker at beginning for stepped maps
                 return tuple([cmap[0]] + list(cmap[-1:0:-1]))
         return cmap
+
+    def formatValsWithDatatypeToText(self, vals, displaydatatype):
+        """Given a set of values, datatype, return a list of strings
+        corresponding to these data."""
+
+        if displaydatatype == 'text':
+            return vals
+        elif displaydatatype == 'numeric':
+            return [ utils.formatNumber(val, '%Vg', locale=self.locale)
+                     for val in vals ]
+        elif displaydatatype == 'date':
+            return [ utils.dateFloatToString(val) for val in vals ]
+        else:
+            raise RuntimeError('Invalid data type')
