@@ -26,25 +26,26 @@ Operations store paths to objects to be modified rather than object references
 because some operations cannot restore references (e.g. add object)
 """
 
+from __future__ import division, print_function
 import os.path
-from itertools import izip
+import codecs
 
 import numpy as N
 
-import datasets
-import widgetfactory
-import simpleread
-import readcsv
-import linked
+from ..compat import czip, crange, citems, cbasestr
+from . import datasets
+from . import widgetfactory
+from . import simpleread
+from . import readcsv
+from . import linked
 
-import veusz.utils as utils
-import veusz.plugins as plugins
-import veusz.qtall as qt4
+from .. import utils
+from .. import plugins
+from .. import qtall as qt4
 
 def _(text, disambiguation=None, context="Operations"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 ###############################################################################
 # Setting operations
@@ -59,7 +60,7 @@ class OperationSettingSet(object):
         Setting may be a widget path
         """
         
-        if isinstance(setting, basestring):
+        if isinstance(setting, cbasestr):
             self.settingpath = setting
         else:
             self.settingpath = setting.path
@@ -142,7 +143,7 @@ class OperationSettingPropagate(object):
     def undo(self, document):
         """Undo all those changes."""
         
-        for setpath, setval in self.restorevals.iteritems():
+        for setpath, setval in citems(self.restorevals):
             setting = document.resolveFullSettingPath(setpath)
             setting.set(setval)
 
@@ -237,11 +238,11 @@ class OperationWidgetsDelete(object):
         # ignore widgets which share ancestry
         # as deleting the parent deletes the child
         widgetpaths = list(self.widgetpaths)
-        widgetpaths.sort( cmp=lambda a, b: len(a)-len(b) )
+        widgetpaths.sort(key=len)
         i = 0
         while i < len(widgetpaths):
             wp = widgetpaths[i]
-            for j in xrange(i):
+            for j in crange(i):
                 if wp[:len(widgetpaths[j])+1] == widgetpaths[j]+'/':
                     del widgetpaths[i]
                     break
@@ -264,7 +265,7 @@ class OperationWidgetsDelete(object):
         """Restore deleted widget."""
         
         # put back widgets in reverse order so that indexes are corrent
-        for i in xrange(len(self.oldwidgets)-1,-1,-1):
+        for i in crange(len(self.oldwidgets)-1,-1,-1):
             oldparent = document.resolveFullWidgetPath(self.oldparentpaths[i])
             oldparent.addChild(self.oldwidgets[i], index=self.oldindexes[i])
         
@@ -659,20 +660,16 @@ class OperationDatasetCreateExpression(OperationDatasetCreate):
 
     def validateExpression(self, document):
         """Validate the expression is okay.
-        A CreateDatasetException is raised if not
+
+        Returns True if ok
         """
 
         p = self.parts.copy()
         p['parametric'] = self.parametric
         ds = datasets.DatasetExpression(**p)
         ds.document = document
-        try:
-            # we force an evaluation of the dataset for the first time, to
-            # check for errors in the expressions
-            ds.updateEvaluation()
-            
-        except datasets.DatasetExpressionException, e:
-            raise CreateDatasetException(unicode(e))
+
+        return ds.updateEvaluation()
         
     def do(self, document):
         """Create the dataset."""
@@ -701,13 +698,11 @@ class OperationDataset2DBase(object):
     
     def validateExpression(self, document):
         """Validate expression is okay."""
-        try:
-            ds = self.makeDSClass()
-            ds.document = document
-            ds.evalDataset()
-            
-        except datasets.DatasetExpressionException, e:
-            raise CreateDatasetException(unicode(e))
+        ds = self.makeDSClass()
+        ds.document = document
+        ds.evalDataset()
+        if 0 in ds.data.shape:
+            raise CreateDatasetException()
 
     def do(self, document):
         """Make new dataset."""
@@ -784,14 +779,14 @@ class OperationDatasetUnlinkByFile(object):
     def do(self, document):
         """Remove links."""
         self.oldlinks = {}
-        for name, ds in document.data.iteritems():
+        for name, ds in citems(document.data):
             if ds.linked is not None and ds.linked.filename == self.filename:
                 self.oldlinks[name] = ds.linked
                 ds.linked = None
 
     def undo(self, document):
         """Restore links."""
-        for name, link in self.oldlinks.iteritems():
+        for name, link in citems(self.oldlinks):
             try:
                 document.data[name].linked = link
             except KeyError:
@@ -809,14 +804,14 @@ class OperationDatasetDeleteByFile(object):
     def do(self, document):
         """Remove datasets."""
         self.olddatasets = {}
-        for name, ds in document.data.items():
+        for name, ds in list(document.data.items()):
             if ds.linked is not None and ds.linked.filename == self.filename:
                 self.olddatasets[name] = ds
                 document.deleteData(name)
 
     def undo(self, document):
         """Restore datasets."""
-        for name, ds in self.olddatasets.iteritems():
+        for name, ds in citems(self.olddatasets):
             document.setData(name, ds)
 
 ###############################################################################
@@ -913,7 +908,7 @@ class OperationDataImport(OperationDataImportBase):
         elif p.datastr is not None:
             stream = simpleread.StringStream(p.datastr)
         else:
-            raise RuntimeError, "No filename or string"
+            raise RuntimeError("No filename or string")
 
         # do the import
         self.simpleread.clearState()
@@ -1018,7 +1013,7 @@ class OperationDataImportFITS(OperationDataImportBase):
         if ( p.datacol is not None or p.symerrcol is not None
              or p.poserrcol is not None
              or p.negerrcol is not None ):
-            print "Warning: ignoring columns as import 2D dataset"
+            print("Warning: ignoring columns as import 2D dataset")
 
         header = hdu.header
         data = hdu.data
@@ -1055,10 +1050,13 @@ class OperationDataImportFITS(OperationDataImportBase):
         """Do the import."""
 
         try:
-            import pyfits
+            from astropy.io import fits as pyfits
         except ImportError:
-            raise RuntimeError, ( 'PyFITS is required to import '
-                                  'data from FITS files' )
+            try:
+                import pyfits
+            except ImportError:
+                raise RuntimeError('Either Astropy or PyFITS is required '
+                                   'to import data from FITS files')
 
         p = self.params
         f = pyfits.open( str(p.filename), 'readonly')
@@ -1076,7 +1074,7 @@ class OperationDataImportFITS(OperationDataImportBase):
             elif naxis == 2:
                 ds = self._import2dimage(hdu)
             else:
-                raise RuntimeError, "Cannot import images with %i dimensions" % naxis
+                raise RuntimeError("Cannot import images with %i dimensions" % naxis)
         f.close()
 
         if p.linked:
@@ -1085,7 +1083,7 @@ class OperationDataImportFITS(OperationDataImportBase):
             self.olddataset = document.data[p.dsname]
         else:
             self.olddataset = None
-        document.setData(p.dsname, ds)
+        document.setData(p.dsname.strip(), ds)
         self.outdatasets.append(p.dsname)
 
 class OperationDataImportPlugin(OperationDataImportBase):
@@ -1266,7 +1264,7 @@ class OperationDatasetAddColumn(object):
             setattr(ds, self.columnname,
                     N.zeros(datacol.shape, dtype='float64'))
         except AttributeError:
-            raise RuntimeError, "Invalid column name for dataset"
+            raise RuntimeError("Invalid column name for dataset")
         document.setData(self.datasetname, ds)
 
     def undo(self, document):
@@ -1439,7 +1437,7 @@ class OperationLoadStyleSheet(OperationMultiple):
     def do(self, document):
         """Do the import."""
 
-        import commandinterpreter
+        from . import commandinterpreter
 
         # get document to keep track of changes for undo/redo
         document.batchHistory(self)
@@ -1447,7 +1445,8 @@ class OperationLoadStyleSheet(OperationMultiple):
         # fire up interpreter to read file
         interpreter = commandinterpreter.CommandInterpreter(document)
         try:
-            interpreter.runFile( open(self.filename) )
+            interpreter.runFile( codecs.open(self.filename, 'rU',
+                                             encoding='utf8') )
         except:
             document.batchHistory(None)
             raise
@@ -1468,7 +1467,7 @@ class OperationToolsPlugin(OperationMultiple):
     def do(self, document):
         """Use the plugin."""
 
-        import commandinterface
+        from . import commandinterface
 
         # get document to keep track of changes for undo/redo
         document.batchHistory(self)
@@ -1505,7 +1504,7 @@ class OperationDatasetPlugin(object):
         names = self.datasetnames = list(manager.datasetnames)
 
         # rename if requested
-        for i in xrange(len(names)):
+        for i in crange(len(names)):
             if names[i] in self.names:
                 names[i] = self.names[names[i]]
 
@@ -1515,7 +1514,7 @@ class OperationDatasetPlugin(object):
                 self.olddata[name] = document.data[name]
 
         # add new datasets to document
-        for name, ds in izip(names, manager.veuszdatasets):
+        for name, ds in czip(names, manager.veuszdatasets):
             if name is not None:
                 document.setData(name, ds)
 
@@ -1534,5 +1533,5 @@ class OperationDatasetPlugin(object):
                 document.deleteData(name)
 
         # put back old datasets
-        for name, ds in self.olddata.iteritems():
+        for name, ds in citems(self.olddata):
             document.setData(name, ds)
