@@ -40,13 +40,14 @@ Commas are now optional in 1.6, so descriptors can look like
 x +- y + -
 """
 
+from __future__ import division
 import re
-import cStringIO
 
 import numpy as N
 
-import veusz.utils as utils
-import datasets
+from ..compat import crange, cnext, CStringIO, citems
+from .. import utils
+from . import datasets
 
 # a regular expression for splitting descriptor into tokens
 descrtokens_split_re = re.compile(r'''
@@ -221,7 +222,7 @@ class DescriptorPart(object):
         """Read data from stream, and write to thedatasets."""
 
         # loop over column range
-        for index in xrange(self.startindex, self.stopindex+1):
+        for index in crange(self.startindex, self.stopindex+1):
             # name for variable
             if self.single:
                 name = self.name
@@ -288,7 +289,7 @@ class DescriptorPart(object):
             return []
 
         names = []
-        for index in xrange(self.startindex, self.stopindex+1):
+        for index in crange(self.startindex, self.stopindex+1):
             # name for variable
             if self.single:
                 name = '%s' % (self.name,)
@@ -335,7 +336,7 @@ class DescriptorPart(object):
                     ds = datasets.DatasetText( data=vals,
                                                linked = linkedfile )
                 else:
-                    raise RuntimeError, "Invalid data type"
+                    raise RuntimeError("Invalid data type")
 
                 finalname = prefix + name + suffix
                 document.setData( finalname, ds )
@@ -420,7 +421,7 @@ class FileStream(Stream):
     def readLine(self):
         """Read the next line of the data source.
         StopIteration is raised if there is no more data."""
-        return self.file.next()
+        return cnext(self.file)
 
 class StringStream(FileStream):
     '''For reading data from a string.'''
@@ -428,7 +429,7 @@ class StringStream(FileStream):
     def __init__(self, text):
         """A stream which reads in from a text string."""
         
-        FileStream.__init__( self, cStringIO.StringIO(text) )
+        FileStream.__init__( self, CStringIO(text) )
 
 class SimpleRead(object):
     '''Class to read in datasets from a stream.
@@ -550,7 +551,7 @@ class SimpleRead(object):
             stream.flushLine()
 
         self.parts = allparts
-        self.blocks = blocks.keys()
+        self.blocks = list(blocks.keys())
 
     def getInvalidConversions(self):
         """Return the number of invalid conversions after reading data.
@@ -566,7 +567,7 @@ class SimpleRead(object):
         """Get a dict of the datasets read (main data part) and number
         of entries read."""
         out = {}
-        for name, data in self.datasets.iteritems():
+        for name, data in citems(self.datasets):
             if name[-2:] == '\0D':
                 out[name[:-2]] = len(data)
         return out
@@ -621,13 +622,13 @@ class SimpleRead2D(object):
         try:
             self.params.xrange = ( float(cols[1]), float(cols[2]) )
         except ValueError:
-            raise Read2DError, "Could not interpret xrange"
+            raise Read2DError("Could not interpret xrange")
         
     def _paramYRange(self, cols):
         try:
             self.params.yrange = ( float(cols[1]), float(cols[2]) )
         except ValueError:
-            raise Read2DError, "Could not interpret yrange"
+            raise Read2DError("Could not interpret yrange")
 
     def _paramInvertRows(self, cols):
         self.params.invertrows = True
@@ -690,7 +691,7 @@ class SimpleRead2D(object):
                 try:
                     line.append( float(v) )
                 except ValueError:
-                    raise Read2DError, "Could not interpret number '%s'" % v
+                    raise Read2DError("Could not interpret number '%s'" % v)
 
             # add row to dataset
             if len(line) != 0:
@@ -704,17 +705,17 @@ class SimpleRead2D(object):
 
         # dodgy formatting probably...
         if len(rows) == 0:
-            raise Read2DError, "No data could be imported for dataset"
+            raise Read2DError("No data could be imported for dataset")
 
         # convert the data to a numpy
         try:
             self.data = N.array(rows)
         except ValueError:
-            raise Read2DError, "Could not convert data to 2D matrix"
+            raise Read2DError("Could not convert data to 2D matrix")
 
         # obvious check
         if len(self.data.shape) != 2:
-            raise Read2DError, "Dataset was not 2D"
+            raise Read2DError("Dataset was not 2D")
 
         # transpose matrix if requested
         if self.params.transpose:
