@@ -20,21 +20,21 @@
 
 """Plugins for creating datasets."""
 
+from __future__ import division
 import numpy as N
-from itertools import izip
-import field
+from . import field
 
-import veusz.utils as utils
+from ..compat import czip, citems, cstr
+from .. import utils
 try:
-    import veusz.helpers.qtloops as qtloops
+    from ..helpers import qtloops
 except ImportError:
     pass
-import veusz.qtall as qt4
+from .. import qtall as qt4
 
 def _(text, disambiguation=None, context='DatasetPlugin'):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # add an instance of your class to this list to be registered
 datasetpluginregistry = []
@@ -84,7 +84,7 @@ class Dataset1D(object):
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
         # need to do the import here as otherwise we get a loop
-        import veusz.document as document
+        from .. import document
         return document.Dataset1DPlugin(manager, self)
 
 class Dataset2D(object):
@@ -111,7 +111,7 @@ class Dataset2D(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        import veusz.document as document
+        from .. import document
         return document.Dataset2DPlugin(manager, self)
 
 class DatasetDateTime(object):
@@ -149,8 +149,8 @@ class DatasetDateTime(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        import veusz.document as document
-        return document.DatasetDatePlugin(manager, self)
+        from .. import document
+        return document.DatasetDateTimePlugin(manager, self)
 
 class DatasetText(object):
     """Text dataset for ImportPlugin or DatasetPlugin."""
@@ -171,7 +171,7 @@ class DatasetText(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        import veusz.document as document
+        from .. import document
         return document.DatasetTextPlugin(manager, self)
 
 class Constant(object):
@@ -205,28 +205,33 @@ class DatasetPluginHelper(object):
     @property
     def datasets1d(self):
         """Return list of existing 1D numeric datasets"""
-        return [name for name, ds in self._doc.data.iteritems() if
+        return [name for name, ds in citems(self._doc.data) if
                 (ds.dimensions == 1 and ds.datatype == 'numeric')]
 
     @property
     def datasets2d(self):
         """Return list of existing 2D numeric datasets"""
-        return [name for name, ds in self._doc.data.iteritems() if
+        return [name for name, ds in citems(self._doc.data) if
                 (ds.dimensions == 2 and ds.datatype == 'numeric')]
 
     @property
     def datasetstext(self):
         """Return list of existing 1D text datasets"""
-        return [name for name, ds in self._doc.data.iteritems() if
+        return [name for name, ds in citems(self._doc.data) if
                 (ds.dimensions == 1 and ds.datatype == 'text')]
 
     @property
     def datasetsdatetime(self):
         """Return list of existing date-time datesets"""
-        import veusz.document as document
-        return [name for name, ds in self._doc.data.iteritems() if
+        from .. import document
+        return [name for name, ds in citems(self._doc.data) if
                 isinstance(ds, document.DatasetDateTime)]
 
+    @property
+    def locale(self):
+        """Return Qt locale."""
+        return self._doc.locale
+
     def evaluateExpression(self, expr, part='data'):
         """Return results of evaluating a 1D dataset expression.
         part is 'data', 'serr', 'perr' or 'nerr' - these are the
@@ -250,7 +255,7 @@ class DatasetPluginHelper(object):
         name not found: raise a DatasetPluginException
         dimensions not right: raise a DatasetPluginException
         """
-        import veusz.document as document
+        from .. import document
         try:
             ds = self._doc.data[name]
         except KeyError:
@@ -345,7 +350,7 @@ class DatasetPluginManager(object):
 
         # look for renamed or deleted datasets
         names = {}
-        for ds, dsname in izip( self.veuszdatasets, self.datasetnames ):
+        for ds, dsname in czip( self.veuszdatasets, self.datasetnames ):
             try:
                 currentname = self.document.datasetName(ds)
             except ValueError:
@@ -374,13 +379,13 @@ class DatasetPluginManager(object):
         # run the plugin with its parameters
         try:
             self.plugin.updateDatasets(self.fields, self.helper)
-        except DatasetPluginException, ex:
+        except DatasetPluginException as ex:
             # this is for immediate notification
             if raiseerrors:
                 raise
 
             # otherwise if there's an error, then log and null outputs
-            self.document.log( unicode(ex) )
+            self.document.log( cstr(ex) )
             self.nullDatasets()
 
 class DatasetPlugin(object):
@@ -771,7 +776,7 @@ class PartsDatasetPlugin(DatasetPlugin):
         ds_in = helper.getDataset(fields['ds_in'])
         data, serr, perr, nerr = ds_in.data, ds_in.serr, ds_in.perr, ds_in.nerr
 
-        plen = float(len(data)) / len(self.dsout)
+        plen = len(data) / len(self.dsout)
         for i, ds in enumerate(self.dsout):
             minv, maxv = int(plen*i), int(plen*(i+1))
             pserr = pperr = pnerr = None
@@ -1426,9 +1431,9 @@ class FilterDatasetPlugin(_OneOutputDatasetPlugin):
                 if serr is not None: serr = serr[filt]
                 if perr is not None: perr = perr[filt]
                 if nerr is not None: nerr = nerr[filt]
-        except (ValueError, IndexError), e:
+        except (ValueError, IndexError) as e:
             raise DatasetPluginException(_("Error filtering dataset: '%s')") %
-                                         unicode(e))
+                                         cstr(e))
 
         self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
 
@@ -1504,6 +1509,112 @@ class LinearInterpolatePlugin(_OneOutputDatasetPlugin):
 
         self.dsout.update(data=interpol)
 
+class ReBinXYPlugin(DatasetPlugin):
+    """Bin-up data by factor given."""
+
+    menu = (_('Filtering'), _('Bin X,Y'))
+    name = 'RebinXY'
+    description_short = 'Bin every N datapoints'
+    description_full = ('Given dataset Y (and optionally X), for every N '
+                        'datapoints calculate the binned value. For '
+                        'dataset Y this is the sum or mean of every N '
+                        'datapoints. For X this is the midpoint of the '
+                        'datapoints (using error bars to give the range.')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_y', _('Input dataset Y')),
+            field.FieldDataset('ds_x', _('Input dataset X (optional)')),
+            field.FieldInt('binsize', _('Bin size (N)'),
+                           minval=1, default=2),
+            field.FieldCombo('mode', _('Mode of binning'),
+                             items=('sum', 'average'),
+                             editable=False),
+            field.FieldDataset('ds_yout', _("Output Y'")),
+            field.FieldDataset('ds_xout', _("Output X' (optional)")),
+            ]
+
+    def getDatasets(self, fields):
+        """Return output datasets"""
+        if fields['ds_yout'] == '':
+            raise DatasetPluginException('Invalid output Y dataset name')
+        if fields['ds_x'] != '' and fields['ds_xout'] == '':
+            raise DatasetPluginException('Invalid output X dataset name')
+
+        self.dssout = out = [ Dataset1D(fields['ds_yout']) ]
+        if fields['ds_xout'] != '':
+            out.append(Dataset1D(fields['ds_xout']))
+        return out
+
+    def updateDatasets(self, fields, helper):
+        """Do binning."""
+
+        binsize = fields['binsize']
+        average = fields['mode'] == 'average'
+
+        def binerr(err):
+            """Compute binned error."""
+            if err is None:
+                return None
+            err2 = qtloops.binData(err**2, binsize, False)
+            cts = qtloops.binData(N.ones(err.shape), binsize, False)
+            return N.sqrt(err2) / cts
+
+        # bin up data and calculate errors (if any)
+        dsy = helper.getDataset(fields['ds_y'])
+        binydata = qtloops.binData(dsy.data, binsize, average)
+        binyserr = binerr(dsy.serr)
+        binyperr = binerr(dsy.perr)
+        binynerr = binerr(dsy.nerr)
+
+        self.dssout[0].update(data=binydata, serr=binyserr, perr=binyperr, nerr=binynerr)
+
+        if len(self.dssout) == 2:
+            # x datasets
+            dsx = helper.getDataset(fields['ds_x'])
+
+            # Calculate ranges between adjacent binned points.  This
+            # is horribly messy - we have to account for the fact
+            # there might not be error bars and calculate the midpoint
+            # to the previous/next point.
+            minvals = N.array(dsx.data)
+            if dsx.serr is not None:
+                minvals -= dsx.serr
+            elif dsx.nerr is not None:
+                minvals += dsx.nerr
+            else:
+                minvals = 0.5*(dsx.data[1:] + dsx.data[:-1])
+                if len(dsx.data) > 2:
+                    # assume +ve error bar on last point is as big as its -ve error
+                    minvals = N.insert(minvals, 0, dsx.data[0] - 0.5*(
+                            dsx.data[1] - dsx.data[0]))
+                elif len(dsx.data) != 0:
+                    # no previous point so we assume 0 error
+                    minvals = N.insert(minvals, 0, dsx.data[0])
+
+            maxvals = N.array(dsx.data)
+            if dsx.serr is not None:
+                maxvals += dsx.serr
+            elif dsx.perr is not None:
+                maxvals += dsx.perr
+            else:
+                maxvals = 0.5*(dsx.data[1:] + dsx.data[:-1])
+                if len(dsx.data) > 2:
+                    maxvals = N.append(maxvals, dsx.data[-1] + 0.5*(
+                            dsx.data[-1] - dsx.data[-2]))
+                elif len(dsx.data) != 0:
+                    maxvals = N.append(maxvals, dsx.data[-1])
+
+            minbin = minvals[::binsize]
+            maxbin = maxvals[binsize-1::binsize]
+            if len(minbin) > len(maxbin):
+                # not an even number of bin size
+                maxbin = N.append(maxbin, maxvals[-1])
+
+            self.dssout[1].update(data=0.5*(minbin+maxbin),
+                                  serr=0.5*(maxbin-minbin))
+
 class SortPlugin(_OneOutputDatasetPlugin):
     """Sort a dataset."""
 
@@ -1589,7 +1700,7 @@ class Histogram2D(DatasetPlugin):
     menu = (_('Compute'), _('2D histogram'),)
     name = 'Histogram 2D'
     description_short = _('Compute 2D histogram.')
-    description_long = _('Given two 1D datasets, compute a 2D histogram. '
+    description_full = _('Given two 1D datasets, compute a 2D histogram. '
                          'Can optionally compute a probability distribution.')
 
     def __init__(self):
@@ -1683,6 +1794,45 @@ class Histogram2D(DatasetPlugin):
         self.dsout = Dataset2D(fields['ds_out'])
         return [self.dsout]
 
+class ConvertNumbersToText(DatasetPlugin):
+    """Convert a set of numbers to text."""
+
+    menu = (_('Convert'), _('Numbers to Text'),)
+    name = 'NumbersToText'
+    description_short = _('Convert numeric dataset to text')
+    description_full = _('Given a 1D numeric dataset, create a text dataset '
+                         'by applying formatting. Format string is in standard '
+                         'Veusz-extended C formatting, e.g.\n'
+                         ' "%Vg" - general,'
+                         ' "%Ve" - scientific,'
+                         ' "%VE" - engineering suffix,'
+                         ' "%.2f" - two decimal places and'
+                         ' "%e" - C-style scientific')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', _('Input dataset')),
+            field.FieldText('format', _('Format'), default='%Vg'),
+            field.FieldDataset('ds_out', _('Output dataset name')),
+            ]
+
+    def getDatasets(self, fields):
+        if fields['ds_out'] == '':
+            raise DatasetPluginException('Invalid output dataset name')
+        self.dsout = DatasetText(fields['ds_out'])
+        return [self.dsout]
+
+    def updateDatasets(self, fields, helper):
+        """Convert dataset."""
+        ds_in = helper.getDataset(fields['ds_in'])
+        f = fields['format']
+
+        data = [ utils.formatNumber(n, f, locale=helper.locale)
+                 for n in ds_in.data ]
+
+        self.dsout.update(data=data)
+
 datasetpluginregistry += [
     AddDatasetPlugin,
     AddDatasetsPlugin,
@@ -1707,11 +1857,14 @@ datasetpluginregistry += [
     ThinDatasetPlugin,
 
     PolarToCartesianPlugin,
+    ConvertNumbersToText,
 
     FilterDatasetPlugin,
 
     MovingAveragePlugin,
     LinearInterpolatePlugin,
+    ReBinXYPlugin,
+
     SortPlugin,
     SortTextPlugin,
 
