@@ -19,22 +19,23 @@
 
 """A widget for navigating datasets."""
 
+from __future__ import division
 import os.path
 import numpy as N
 import textwrap
 
-import veusz.qtall as qt4
-import veusz.setting as setting
-import veusz.document as document
-import veusz.utils as utils
+from ..compat import crange, citems, czip
+from .. import qtall as qt4
+from .. import setting
+from .. import document
+from .. import utils
 
-from lineeditwithclear import LineEditWithClear
-from veusz.utils.treemodel import TMNode, TreeModel
+from .lineeditwithclear import LineEditWithClear
+from ..utils.treemodel import TMNode, TreeModel
 
 def _(text, disambiguation=None, context="DatasetBrowser"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 def datasetLinkFile(ds):
     """Get a linked filename from a dataset."""
@@ -80,7 +81,7 @@ class DatasetNode(TMNode):
             if len(ds.data) < size[1]:
                 y = ds.data
             else:
-                intvl = len(ds.data)/size[1]+1
+                intvl = len(ds.data)//size[1]+1
                 y = ds.data[::intvl]
             x = N.arange(len(y))
 
@@ -118,7 +119,7 @@ class DatasetNode(TMNode):
         try:
             ds = self.doc.data[self.data[0]]
         except KeyError:
-            return qt4.QVariant()
+            return None
 
         c = self.cols[column]
         if c == "name":
@@ -127,9 +128,9 @@ class DatasetNode(TMNode):
                 text = ''
 
             if ds.tags:
-                text += '\n\n' + _('Tags: %s') % (' '.join(list(sorted(ds.tags))))
+                text += '\n\n' + _('Tags: %s') % (' '.join(sorted(ds.tags)))
 
-            return qt4.QVariant(textwrap.fill(text, 40))
+            return textwrap.fill(text, 40)
         elif c == "size" or (c == 'type' and 'size' not in self.cols):
             text = ds.userPreview()
             # add preview of dataset if possible
@@ -137,10 +138,10 @@ class DatasetNode(TMNode):
             if pix:
                 text = text.replace("\n", "<br>")
                 text = "<html>%s<br>%s</html>" % (text, utils.pixmapAsHtml(pix))
-            return qt4.QVariant(text)
+            return text
         elif c == "linkfile" or c == "type":
-            return qt4.QVariant(textwrap.fill(ds.linkedInformation(), 40))
-        return qt4.QVariant()
+            return textwrap.fill(ds.linkedInformation(), 40)
+        return None
 
     def dataset(self):
         """Get associated dataset."""
@@ -164,10 +165,10 @@ class FilenameNode(TMNode):
         """basename of filename for data."""
         if column == 0:
             if self.data[0] == "/":
-                return qt4.QVariant("/")
+                return "/"
             else:
-                return qt4.QVariant(os.path.basename(self.data[0]))
-        return qt4.QVariant()
+                return os.path.basename(self.data[0])
+        return None
 
     def filename(self):
         """Return filename."""
@@ -176,8 +177,8 @@ class FilenameNode(TMNode):
     def toolTip(self, column):
         """Full filename for tooltip."""
         if column == 0:
-            return qt4.QVariant(self.data[0])
-        return qt4.QVariant()
+            return self.data[0]
+        return None
 
 def treeFromList(nodelist, rootdata):
     """Construct a tree from a list of nodes."""
@@ -207,7 +208,7 @@ class DatasetRelationModel(TreeModel):
         self.filterdtype = filterdtype
         self.refresh()
 
-        self.connect(doc, qt4.SIGNAL("sigModified"), self.refresh)
+        doc.signalModified.connect(self.refresh)
 
     def datasetFilterOut(self, ds, node):
         """Should dataset be filtered out by filter options."""
@@ -237,7 +238,7 @@ class DatasetRelationModel(TreeModel):
     def makeGrpTreeNone(self):
         """Make tree with no grouping."""
         tree = TMNode( (_("Dataset"), _("Size"), _("Type"), _("File")), None )
-        for name, ds in self.doc.data.iteritems():
+        for name, ds in citems(self.doc.data):
             child = DatasetNode( self.doc, name,
                                  ("name", "size", "type", "linkfile"),
                                  None )
@@ -255,7 +256,7 @@ class DatasetRelationModel(TreeModel):
         GrpNodeClass: class for creating grouping nodes
         """
         grpnodes = {}
-        for name, ds in self.doc.data.iteritems():
+        for name, ds in citems(self.doc.data):
             child = DatasetNode(self.doc, name, colitems, None)
 
             # check whether filtered out
@@ -268,7 +269,7 @@ class DatasetRelationModel(TreeModel):
                     # add to group
                     grpnodes[grp].insertChildSorted(child)
 
-        return treeFromList(grpnodes.values(), coltitles)
+        return treeFromList(list(grpnodes.values()), coltitles)
 
     def makeGrpTreeFilename(self):
         """Make a tree of datasets grouped by linked file."""
@@ -302,7 +303,7 @@ class DatasetRelationModel(TreeModel):
 
         def getgrp(ds):
             if ds.tags:
-                return list(sorted(ds.tags))
+                return sorted(ds.tags)
             else:
                 return [_("None")]
 
@@ -322,10 +323,9 @@ class DatasetRelationModel(TreeModel):
             f |= qt4.Qt.ItemIsEditable
         return f
 
-    def setData(self, idx, data, role):
+    def setData(self, idx, newname, role):
         """Rename dataset."""
         dsnode = self.objFromIndex(idx)
-        newname = unicode(data.toString())
         if not utils.validateDatasetName(newname) or newname in self.doc.data:
             return False
 
@@ -385,7 +385,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
         hdr = self.header()
         hdr.setStretchLastSection(False)
         hdr.setResizeMode(0, qt4.QHeaderView.Stretch)
-        for col in xrange(1, 3):
+        for col in crange(1, 3):
             hdr.setResizeMode(col, qt4.QHeaderView.ResizeToContents)
 
         # when documents have finished opening, expand all nodes
@@ -420,7 +420,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
 
         matches = self.model.match(
             self.model.index(0, 0, qt4.QModelIndex()),
-            qt4.Qt.DisplayRole, qt4.QVariant(dsname), -1,
+            qt4.Qt.DisplayRole, dsname, -1,
             qt4.Qt.MatchFixedString | qt4.Qt.MatchCaseSensitive |
             qt4.Qt.MatchRecursive )
         for idx in matches:
@@ -460,12 +460,12 @@ class DatasetsNavigatorTree(qt4.QTreeView):
 
         def _paste():
             """Paste dataset(s)."""
-            if document.isDataMime():
+            if document.isClipboardDataMime():
                 mime = qt4.QApplication.clipboard().mimeData()
                 self.doc.applyOperation(document.OperationDataPaste(mime))
 
         # if there is data to paste, add menu item
-        if document.isDataMime():
+        if document.isClipboardDataMime():
             menu.addAction(_("Paste"), _paste)
 
         if len( menu.actions() ) != 0:
@@ -473,20 +473,20 @@ class DatasetsNavigatorTree(qt4.QTreeView):
 
     def datasetContextMenu(self, dsnodes, menu):
         """Return context menu for datasets."""
-        import veusz.dialogs.dataeditdialog as dataeditdialog
+        from ..dialogs import dataeditdialog
 
         datasets = [d.dataset() for d in dsnodes]
         dsnames = [d.datasetName() for d in dsnodes]
 
         def _edit():
             """Open up dialog box to recreate dataset."""
-            for dataset, dsname in zip(datasets, dsnames):
+            for dataset, dsname in czip(datasets, dsnames):
                 if type(dataset) in dataeditdialog.recreate_register:
                     dataeditdialog.recreate_register[type(dataset)](
                         self.mainwindow, self.doc, dataset, dsname)
         def _edit_data():
             """Open up data edit dialog."""
-            for dataset, dsname in zip(datasets, dsnames):
+            for dataset, dsname in czip(datasets, dsnames):
                 if type(dataset) not in dataeditdialog.recreate_register:
                     self.mainwindow.slotDataEdit(editdataset=dsname)
         def _delete():
@@ -500,7 +500,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
             self.doc.applyOperation(
                 document.OperationMultiple(
                     [document.OperationDatasetUnlinkFile(n)
-                     for d,n in zip(datasets,dsnames)
+                     for d,n in czip(datasets,dsnames)
                      if d.canUnlink() and d.linked],
                     descr=_('unlink dataset(s)')))
         def _unlink_relation():
@@ -508,7 +508,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
             self.doc.applyOperation(
                 document.OperationMultiple(
                     [document.OperationDatasetUnlinkRelation(n)
-                     for d,n in zip(datasets,dsnames)
+                     for d,n in czip(datasets,dsnames)
                      if d.canUnlink() and not d.linked],
                     descr=_('unlink dataset(s)')))
         def _copy():
@@ -555,7 +555,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
             tag, ok = qt4.QInputDialog.getText(
                 self, _("New tag"), _("Enter new tag"))
             if ok:
-                tag = unicode(tag).strip().replace(' ', '')
+                tag = tag.strip().replace(' ', '')
                 if tag:
                     self.doc.applyOperation( document.OperationDataTag(
                             tag, dsnames) )
@@ -571,7 +571,7 @@ class DatasetsNavigatorTree(qt4.QTreeView):
     def filenameContextMenu(self, nodes, menu):
         """Return context menu for filenames."""
 
-        from veusz.dialogs.reloaddata import ReloadData
+        from ..dialogs.reloaddata import ReloadData
 
         filenames = [n.filename() for n in nodes if n.filename() != '/']
         if not filenames:
@@ -723,7 +723,7 @@ class DatasetBrowser(qt4.QWidget):
 
     def slotFilterChanged(self, filtertext):
         """Filtering changed by user."""
-        self.navtree.changeFilter(unicode(filtertext))
+        self.navtree.changeFilter(filtertext)
 
     def selectDataset(self, dsname):
         """Find, and if possible select dataset name."""
