@@ -24,20 +24,19 @@
     changed. The creator should use this to change the setting.
 """
 
-from itertools import izip
+from __future__ import division
 import re
 import numpy as N
 
-import veusz.qtall as qt4
+from ..compat import crange, czip, citems, ckeys
+from .. import qtall as qt4
 
-import setting
-
-import veusz.utils as utils
+from .settingdb import settingdb
+from .. import utils
 
 def _(text, disambiguation=None, context="Setting"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 def styleClear(widget):
     """Return widget to default"""
@@ -46,7 +45,7 @@ def styleClear(widget):
 def styleError(widget):
     """Show error state on widget."""
     widget.setStyleSheet("background-color: " +
-                         setting.settingdb.color('error').name() )
+                         settingdb.color('error').name() )
 
 class DotDotButton(qt4.QPushButton):
     """A button for opening up more complex editor."""
@@ -80,13 +79,13 @@ class Edit(qt4.QLineEdit):
     def validateAndSet(self):
         """Check the text is a valid setting and update it."""
 
-        text = unicode(self.text())
+        text = self.text()
         try:
             val = self.setting.fromText(text)
             styleClear(self)
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
-        except setting.InvalidType:
+        except utils.InvalidType:
             styleError(self)
 
     def onModified(self, mod):
@@ -153,7 +152,7 @@ class _EditBox(qt4.QTextEdit):
         """Tell the calling widget that we are closing, and provide
         the new text."""
 
-        text = unicode(self.toPlainText())
+        text = self.toPlainText()
         text = text.replace('\n', '')
         self.emit(qt4.SIGNAL('closing'), text)
         event.accept()
@@ -194,7 +193,7 @@ class String(qt4.QWidget):
 
         # if button is down and there's no existing popup, bring up a new one
         if on:
-            e = _EditBox( unicode(self.edit.text()),
+            e = _EditBox( self.edit.text(),
                           self.setting.readonly, self.button)
 
             # we get notified with text when the popup closes
@@ -216,13 +215,13 @@ class String(qt4.QWidget):
     def validateAndSet(self):
         """Check the text is a valid setting and update it."""
 
-        text = unicode(self.edit.text())
+        text = self.edit.text()
         try:
             val = self.setting.fromText(text)
             styleClear(self.edit)
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
-        except setting.InvalidType:
+        except utils.InvalidType:
             styleError(self.edit)
 
     def onModified(self, mod):
@@ -335,13 +334,13 @@ class Choice(qt4.QComboBox):
             self.addItems( list(vallist) )
         else:
             # add pixmaps and text to list
-            for icon, text in izip(icons, vallist):
+            for icon, text in czip(icons, vallist):
                 self.addItem(icon, text)
 
         # use tooltip descriptions if requested
         if descriptions is not None:
             for i, descr in enumerate(descriptions):
-                self.setItemData(i, qt4.QVariant(descr), qt4.Qt.ToolTipRole)
+                self.setItemData(i, descr, qt4.Qt.ToolTipRole)
 
         # choose the correct setting
         try:
@@ -374,13 +373,13 @@ class Choice(qt4.QComboBox):
     def slotActivated(self, val):
         """If a different item is chosen."""
 
-        text = unicode(self.currentText())
+        text = self.currentText()
         try:
             val = self.setting.fromText(text)
             styleClear(self)
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
-        except setting.InvalidType:
+        except utils.InvalidType:
             styleError(self)
 
     def onModified(self, mod):
@@ -432,7 +431,7 @@ class FillStyleExtended(ChoiceSwitch):
     def _generateIcons(cls):
         """Generate a list of pixmaps for drop down menu."""
 
-        import collections
+        from . import collections
         brush = collections.BrushExtended("")
         brush.color = 'black'
         brush.patternspacing = '5pt'
@@ -476,23 +475,48 @@ class MultiLine(qt4.QTextEdit):
         if setting.readonly:
             self.setReadOnly(True)
 
+        self.document().contentsChanged.connect(self.onSizeChange)
+        self.document().documentLayout().documentSizeChanged.connect(
+            self.onSizeChange)
+
+        self.heightmin = 0
+        self.heightmax = 2048
+
+        # recalculate size of document to fix size
+        self.document().adjustSize()
+        self.onSizeChange()
+
+    def onSizeChange(self):
+        """Make size match content size."""
+        m = self.contentsMargins()
+        docheight = self.document().size().height() + m.top() + m.bottom()
+        docheight = min(self.heightmax, max(self.heightmin, docheight))
+        self.setFixedHeight(docheight)
+
     def focusOutEvent(self, *args):
         """Allows us to check the contents of the widget."""
         qt4.QTextEdit.focusOutEvent(self, *args)
 
-        text = unicode(self.toPlainText())
+        text = self.toPlainText()
         try:
             val = self.setting.fromText(text)
             styleClear(self)
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
-        except setting.InvalidType:
+        except utils.InvalidType:
             styleError(self)
 
     def onModified(self, mod):
         """called when the setting is changed remotely"""
         self.setPlainText( self.setting.toText() )
 
+class Notes(MultiLine):
+    """For editing notes."""
+
+    def __init__(self, setting, parent):
+        MultiLine.__init__(self, setting, parent)
+        self.setWordWrapMode(qt4.QTextOption.WordWrap)
+
 class Distance(Choice):
     """For editing distance settings."""
 
@@ -517,7 +541,7 @@ class Distance(Choice):
         self.blockSignals(True)
 
         # get current text
-        text = unicode(self.currentText())
+        text = self.currentText()
 
         # get rid of non-numeric things from the string
         num = self.stripnumre.sub('', text)
@@ -543,7 +567,7 @@ class Distance(Choice):
             newitems.insert(0, text)
 
         # get rid of existing items in list (clear doesn't work here)
-        for i in range(self.count()):
+        for i in crange(self.count()):
             self.removeItem(0)
 
         # put new items in and select the correct option
@@ -597,7 +621,7 @@ class Dataset(qt4.QWidget):
         self.datatype = datatype
         self.lastdatasets = None
         self._populateEntries()
-        self.connect(document, qt4.SIGNAL("sigModified"), self.slotModified)
+        document.signalModified.connect(self.slotModified)
 
         layout = qt4.QHBoxLayout()
         layout.setSpacing(0)
@@ -615,8 +639,11 @@ class Dataset(qt4.QWidget):
 
         # get datasets of the correct dimension
         datasets = []
-        for name, ds in self.document.data.iteritems():
-            if ds.dimensions == self.dimensions and ds.datatype == self.datatype:
+        for name, ds in citems(self.document.data):
+            if ds.dimensions == self.dimensions and (
+                ds.datatype == self.datatype or
+                self.datatype == 'all' or
+                ds.datatype in self.datatype ):
                 datasets.append(name)
         datasets.sort()
 
@@ -631,9 +658,9 @@ class Dataset(qt4.QWidget):
     def slotButtonToggled(self, on):
         """Bring up list of datasets."""
         if on:
-            from veusz.qtwidgets.datasetbrowser import DatasetBrowserPopup
+            from ..qtwidgets.datasetbrowser import DatasetBrowserPopup
             d = DatasetBrowserPopup(self.document,
-                                    unicode(self.choice.currentText()),
+                                    self.choice.currentText(),
                                     self.button,
                                     filterdims=set((self.dimensions,)),
                                     filterdtype=set((self.datatype,)) )
@@ -653,8 +680,8 @@ class Dataset(qt4.QWidget):
 class DatasetOrString(Dataset):
     """Allow use to choose a dataset or enter some text."""
 
-    def __init__(self, setting, document, dimensions, datatype, parent):
-        Dataset.__init__(self, setting, document, dimensions, datatype, parent)
+    def __init__(self, setting, document, parent):
+        Dataset.__init__(self, setting, document, 1, 'all', parent)
 
         b = self.textbutton = DotDotButton()
         self.layout().addWidget(b)
@@ -665,7 +692,7 @@ class DatasetOrString(Dataset):
 
         # if button is down and there's no existing popup, bring up a new one
         if on:
-            e = _EditBox( unicode(self.choice.currentText()),
+            e = _EditBox( self.choice.currentText(),
                           self.choice.setting.readonly, self.textbutton)
 
             # we get notified with text when the popup closes
@@ -808,12 +835,12 @@ class LineStyle(Choice):
         """Generate a list of icons for drop down menu."""
 
         # import later for dependency issues
-        import veusz.setting.collections
-        import veusz.document
+        from . import collections
+        from .. import document
 
         icons = []
         size = cls.size
-        setn = veusz.setting.collections.Line('temp')
+        setn = collections.Line('temp')
         setn.get('color').set('black')
         setn.get('width').set('1pt')
         
@@ -821,7 +848,7 @@ class LineStyle(Choice):
             pix = qt4.QPixmap(*size)
             pix.fill()
 
-            ph = veusz.document.PaintHelper( (1, 1) )
+            ph = document.PaintHelper( (1, 1) )
 
             painter = qt4.QPainter(pix)
             painter.setRenderHint(qt4.QPainter.Antialiasing)
@@ -920,13 +947,13 @@ class Color(qt4.QWidget):
         col = qt4.QColorDialog.getColor(self.setting.color(), self)
         if col.isValid():
             # change setting
-            val = unicode( col.name() )
+            val = col.name()
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
     def slotActivated(self, val):
         """A different value is selected."""
         
-        text = unicode(self.combo.currentText())
+        text = self.combo.currentText()
         val = self.setting.fromText(text)
         self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
@@ -957,8 +984,7 @@ class WidgetSelector(Choice):
 
         Choice.__init__(self, setting, True, [], parent)
         self.document = document
-        self.connect(document, qt4.SIGNAL('sigModified'),
-                     self.slotModified)
+        document.signalModified.connect(self.slotModified)
 
     def _populateEntries(self):
         pass
@@ -982,7 +1008,7 @@ class WidgetChoice(WidgetSelector):
         widgets = self.setting.getWidgetList()
 
         # we only need the list of names
-        names = widgets.keys()
+        names = list(widgets.keys())
         names.sort()
 
         utils.populateCombo(self, names)
@@ -1045,8 +1071,8 @@ class ListSet(qt4.QFrame):
         self.setting = setting
         self.controls = []
         self.layout = qt4.QGridLayout(self)
-        self.layout.setMargin( self.layout.margin()/2 )
-        self.layout.setSpacing( self.layout.spacing()/4 )
+        self.layout.setMargin( self.layout.margin()//2 )
+        self.layout.setSpacing( self.layout.spacing()//4 )
 
         # ignore changes if this set
         self.ignorechange = False
@@ -1076,7 +1102,7 @@ class ListSet(qt4.QFrame):
         row = -1
         for row, val in enumerate(self.setting.val):
             cntrls = self.populateRow(row, val)
-            for col in xrange(len(cntrls)):
+            for col in crange(len(cntrls)):
                 self.layout.addWidget(cntrls[col], row, col)
             for c in cntrls:
                 c.show()
@@ -1102,11 +1128,6 @@ class ListSet(qt4.QFrame):
         b.setEnabled( len(self.setting.val) > 0 )
         b.show()
 
-    def polish(self):
-        """Remove tooltip from widget - avoid Qt bugs."""
-        qt4.QVBox.polish(self)
-        qt4.QToolTip.remove(self)
-
     def onAddClicked(self):
         """Add a line style to the list given."""
 
@@ -1182,7 +1203,7 @@ class ListSet(qt4.QFrame):
             for icon in icons:
                 wcombo.addItem(icon, "")
         else:
-            for text, icon in izip(texts, icons):
+            for text, icon in czip(texts, icons):
                 wcombo.addItem(icon, text)
 
         wcombo.setCurrentIndex(values.index(val))
@@ -1277,7 +1298,8 @@ class LineSet(ListSet):
         sender = self.sender()
         row, col = self.identifyPosn(sender)
 
-        text = unicode(sender.text())
+        text = sender.text()
+        from . import setting
         if setting.Distance.isDist(text):
             # valid distance
             styleClear(sender)
@@ -1305,7 +1327,7 @@ class _FillBox(qt4.QScrollArea):
 
         self.extbrush = thesetting.returnBrushExtended(row)
 
-        from veusz.windows.treeeditwindow import SettingsProxySingle, \
+        from ..windows.treeeditwindow import SettingsProxySingle, \
             PropertyList
 
         fbox = self
@@ -1532,7 +1554,7 @@ class MultiSettingWidget(qt4.QWidget):
         """Override this to make an editing widget."""
         return None
 
-    def updateControls():
+    def updateControls(self):
         """Override this to update values in controls."""
         pass
 
@@ -1575,12 +1597,12 @@ class Datasets(MultiSettingWidget):
 
     def readControl(self, control):
         """Get text for control."""
-        return unicode( control.lineEdit().text() )
+        return control.lineEdit().text()
 
     def getDatasets(self):
         """Get applicable datasets (sorted)."""
         datasets = []
-        for name, ds in self.document.data.iteritems():
+        for name, ds in citems(self.document.data):
             if (ds.dimensions == self.dimensions and
                 ds.datatype == self.datatype):
                 datasets.append(name)
@@ -1589,7 +1611,7 @@ class Datasets(MultiSettingWidget):
 
     def updateControls(self):
         """Set values of controls."""
-        for cntrls, val in izip(self.controls, self.setting.val):
+        for cntrls, val in czip(self.controls, self.setting.val):
             cntrls[0].lineEdit().setText(val)
 
     def onModified(self, mod):
@@ -1627,11 +1649,11 @@ class Strings(MultiSettingWidget):
 
     def readControl(self, control):
         """Get text for control."""
-        return unicode( control.text() )
+        return control.text()
 
     def updateControls(self):
         """Set values of controls."""
-        for cntrls, val in izip(self.controls, self.setting.val):
+        for cntrls, val in czip(self.controls, self.setting.val):
             cntrls[0].setText(val)        
 
 class Filename(qt4.QWidget):
@@ -1695,19 +1717,19 @@ class Filename(qt4.QWidget):
             self, title, self.edit.text(), filefilter)
 
         if filename:
-            val = unicode(filename)
+            val = filename
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
     def validateAndSet(self):
         """Check the text is a valid setting and update it."""
 
-        text = unicode(self.edit.text())
+        text = self.edit.text()
         try:
             val = self.setting.fromText(text)
             styleClear(self.edit)
             self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, val )
 
-        except setting.InvalidType:
+        except utils.InvalidType:
             styleError(self.edit)
 
     def onModified(self, mod):
@@ -1744,7 +1766,7 @@ class FontFamily(qt4.QFontComboBox):
 
     def slotActivated(self, val):
         """Update setting if a different item is chosen."""
-        newval = unicode(self.currentText())
+        newval = self.currentText()
         self.emit( qt4.SIGNAL('settingChanged'), self, self.setting, newval )
 
     def onModified(self, mod):
@@ -1765,6 +1787,7 @@ class ErrorStyle(Choice):
                         self._errorstyles, parent,
                         icons=self._icons)
 
+    @classmethod
     def _generateIcons(cls):
         """Generate a list of pixmaps for drop down menu."""
         cls._icons = []
@@ -1782,12 +1805,12 @@ class Colormap(Choice):
     size = (32, 12)
 
     def __init__(self, setn, document, parent):
-        names = sorted(document.colormaps.keys())
+        names = sorted(ckeys(document.colormaps))
 
         icons = Colormap._generateIcons(document, names)
-        setting.controls.Choice.__init__(self, setn, True,
-                                         names, parent,
-                                         icons=icons)
+        Choice.__init__(self, setn, True,
+                        names, parent,
+                        icons=icons)
         self.setIconSize( qt4.QSize(*self.size) )
 
     @classmethod
@@ -1838,5 +1861,5 @@ class AxisBound(Choice):
         """Called if the mode of the axis changes.
         Re-set text as float or date."""
 
-        if unicode(self.currentText()).lower() != 'auto':
+        if self.currentText().lower() != 'auto':
             self.setEditText( self.setting.toText() )
