@@ -26,21 +26,23 @@ s.set(42)
 s.fromText('42')
 """
 
+from __future__ import division
 import re
 import sys
 
 import numpy as N
-import veusz.qtall as qt4
 
-import controls
-from settingdb import settingdb, uilocale
-from reference import Reference
+from ..compat import citems, ckeys, cbasestr, cstr, crepr
+from .. import qtall as qt4
+from . import controls
+from .settingdb import settingdb, uilocale
+from .reference import Reference
 
-import veusz.utils as utils
+from .. import utils
 
-# if invalid type passed to set
-class InvalidType(Exception):
-    pass
+def floattostring(f):
+    """Convert float to string with more precision."""
+    return uilocale.toString(f, 'g', 15)
 
 class Setting(object):
     """A class to store a value with a particular type."""
@@ -143,7 +145,7 @@ class Setting(object):
         if isinstance(self._val, Reference):
             return self._val
         else:
-            raise ValueError, "Setting is not a reference"
+            raise ValueError("Setting is not a reference")
 
     def getStylesheetLink(self):
         """Get text that this setting should default to linked to the
@@ -187,7 +189,7 @@ class Setting(object):
     def fromText(self, text):
         """Convert text to type suitable for setting.
 
-        Raises InvalidType if cannot convert."""
+        Raises utils.InvalidType if cannot convert."""
         return None
 
     def readDefaults(self, root, widgetname):
@@ -257,10 +259,10 @@ class Setting(object):
         if (saveall or not self.isDefault()) and not self.readonly:
             if isinstance(self._val, Reference):
                 return "SetToReference('%s%s', %s)\n" % (rootname, self.name,
-                                                         repr(self._val.value))
+                                                         crepr(self._val.value))
             else:
                 return "Set('%s%s', %s)\n" % ( rootname, self.name,
-                                               repr(self.val) )
+                                               crepr(self.val) )
         else:
             return ''
 
@@ -344,11 +346,13 @@ class Setting(object):
     def safeEvalHelper(self, text):
         """Evaluate an expression, catching naughtiness."""
         try:
-            if utils.checkCode(text) is not None:
-                raise InvalidType
-            return float( eval(text, self.getDocument().eval_context) )
+            comp = self.getDocument().compileCheckedExpression(
+                text)
+            if comp is None:
+                raise utils.InvalidType
+            return float( eval(comp, self.getDocument().eval_context) )
         except:
-            raise InvalidType
+            raise utils.InvalidType
 
 # forward setting to another setting
 class SettingBackwardCompat(Setting):
@@ -416,9 +420,9 @@ class Str(Setting):
     typename = 'str'
 
     def convertTo(self, val):
-        if isinstance(val, basestring):
+        if isinstance(val, cbasestr):
             return val
-        raise InvalidType
+        raise utils.InvalidType
 
     def toText(self):
         return self.val
@@ -429,6 +433,14 @@ class Str(Setting):
     def makeControl(self, *args):
         return controls.String(self, *args)
 
+class Notes(Str):
+    """String for making notes."""
+
+    typename = 'str-notes'
+
+    def makeControl(self, *args):
+        return controls.Notes(self, *args)
+
 # Store bools
 class Bool(Setting):
     """Bool setting."""
@@ -438,7 +450,7 @@ class Bool(Setting):
     def convertTo(self, val):
         if type(val) in (bool, int):
             return bool(val)
-        raise InvalidType
+        raise utils.InvalidType
 
     def toText(self):
         if self.val:
@@ -453,7 +465,7 @@ class Bool(Setting):
         elif t in ('false', '0', 'f', 'n', 'no'):
             return False
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def makeControl(self, *args):
         return controls.Bool(self, *args)
@@ -489,11 +501,11 @@ class Int(Setting):
             if val >= self.minval and val <= self.maxval:
                 return val
             else:
-                raise InvalidType, 'Out of range allowed'
-        raise InvalidType
+                raise utils.InvalidType('Out of range allowed')
+        raise utils.InvalidType
 
     def toText(self):
-        return unicode( uilocale.toString(self.val) )
+        return uilocale.toString(self.val)
 
     def fromText(self, text):
         i, ok = uilocale.toLongLong(text)
@@ -503,7 +515,7 @@ class Int(Setting):
         if i >= self.minval and i <= self.maxval:
             return i
         else:
-            raise InvalidType, 'Out of range allowed'
+            raise utils.InvalidType('Out of range allowed')
 
     def makeControl(self, *args):
         return controls.Int(self, *args)
@@ -512,9 +524,9 @@ def _finiteRangeFloat(f, minval=-1e300, maxval=1e300):
     """Return a finite float in range or raise exception otherwise."""
     f = float(f)
     if not N.isfinite(f):
-        raise InvalidType, 'Finite values only allowed'
+        raise utils.InvalidType('Finite values only allowed')
     if f < minval or f > maxval:
-        raise InvalidType, 'Out of range allowed'
+        raise utils.InvalidType('Out of range allowed')
     return f
 
 # for storing floats
@@ -547,10 +559,10 @@ class Float(Setting):
         if isinstance(val, int) or isinstance(val, float):
             return _finiteRangeFloat(val,
                                      minval=self.minval, maxval=self.maxval)
-        raise InvalidType
+        raise utils.InvalidType
 
     def toText(self):
-        return unicode(uilocale.toString(self.val))
+        return floattostring(self.val)
 
     def fromText(self, text):
         f, ok = uilocale.toDouble(text)
@@ -570,10 +582,10 @@ class FloatOrAuto(Float):
     def convertTo(self, val):
         if type(val) in (int, float):
             return _finiteRangeFloat(val, minval=self.minval, maxval=self.maxval)
-        elif isinstance(val, basestring) and val.strip().lower() == 'auto':
+        elif isinstance(val, cbasestr) and val.strip().lower() == 'auto':
             return None
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def convertFrom(self, val):
         if val is None:
@@ -582,11 +594,11 @@ class FloatOrAuto(Float):
             return Float.convertFrom(self, val)
 
     def toText(self):
-        if self.val is None or (isinstance(self.val, basestring) and
+        if self.val is None or (isinstance(self.val, cbasestr) and
                                 self.val.lower() == 'auto'):
             return 'Auto'
         else:
-            return unicode(uilocale.toString(self.val))
+            return floattostring(self.val)
 
     def fromText(self, text):
         if text.strip().lower() == 'auto':
@@ -605,10 +617,10 @@ class IntOrAuto(Setting):
     def convertTo(self, val):
         if isinstance(val, int):
             return val
-        elif isinstance(val, basestring) and val.strip().lower() == 'auto':
+        elif isinstance(val, cbasestr) and val.strip().lower() == 'auto':
             return None
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def convertFrom(self, val):
         if val is None:
@@ -617,11 +629,11 @@ class IntOrAuto(Setting):
             return val
 
     def toText(self):
-        if self.val is None or (isinstance(self.val, basestring) and
+        if self.val is None or (isinstance(self.val, cbasestr) and
                                 self.val.lower() == 'auto'):
             return 'Auto'
         else:
-            return unicode( uilocale.toString(self.val) )
+            return uilocale.toString(self.val)
 
     def fromText(self, text):
         if text.strip().lower() == 'auto':
@@ -629,7 +641,7 @@ class IntOrAuto(Setting):
         else:
             i, ok = uilocale.toLongLong(text)
             if not ok:
-                raise InvalidType
+                raise utils.InvalidType
             return i
             
     def makeControl(self, *args):
@@ -749,20 +761,20 @@ class Distance(Setting):
         if self.distre.match(val) is not None:
             return val
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def toText(self):
         # convert decimal point to display locale
-        return self.val.replace('.', qt4.QString(uilocale.decimalPoint()))
+        return self.val.replace('.', uilocale.decimalPoint())
 
     def fromText(self, text):
         # convert decimal point from display locale
-        text = text.replace(qt4.QString(uilocale.decimalPoint()), '.')
+        text = text.replace(uilocale.decimalPoint(), '.')
 
         if self.isDist(text):
             return text
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def makeControl(self, *args):
         return controls.Distance(self, *args)
@@ -874,7 +886,7 @@ class Choice(Setting):
         if val in self.vallist:
             return val
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def toText(self):
         return self.val
@@ -883,7 +895,7 @@ class Choice(Setting):
         if text in self.vallist:
             return text
         else:
-            raise InvalidType
+            raise utils.InvalidType
 
     def makeControl(self, *args):
         argsv = {'descriptions': self.descriptions}
@@ -933,22 +945,20 @@ class FloatDict(Setting):
 
     def convertTo(self, val):
         if type(val) != dict:
-            raise InvalidType
+            raise utils.InvalidType
 
         out = {}
-        for key, val in val.iteritems():
+        for key, val in citems(val):
             if type(val) not in (float, int):
-                raise InvalidType
+                raise utils.InvalidType
             else:
                 out[key] = val
 
         return out
 
     def toText(self):
-        keys = self.val.keys()
-        keys.sort()
-        
-        text = ['%s = %s' % (k, uilocale.toString(self.val[k])) for k in keys]
+        text = ['%s = %s' % (k, floattostring(self.val[k]))
+                for k in sorted(ckeys(self.val))]
         return '\n'.join(text)
 
     def fromText(self, text):
@@ -965,11 +975,11 @@ class FloatDict(Setting):
             p = l.strip().split('=')
 
             if len(p) != 2:
-                raise InvalidType
+                raise utils.InvalidType
 
             v, ok = uilocale.toDouble(p[1])
             if not ok:
-                raise InvalidType
+                raise utils.InvalidType
 
             out[ p[0].strip() ] = v
         return out
@@ -984,13 +994,13 @@ class FloatList(Setting):
 
     def convertTo(self, val):
         if type(val) not in (list, tuple):
-            raise InvalidType
+            raise utils.InvalidType
 
         # horribly slow test for invalid entries
         out = []
         for i in val:
             if type(i) not in (float, int):
-                raise InvalidType
+                raise utils.InvalidType
             else:
                 out.append( float(i) )
         return out
@@ -1000,16 +1010,16 @@ class FloatList(Setting):
         # can't use the comma for splitting if used as a decimal point
 
         join = ', '
-        if uilocale.decimalPoint() == qt4.QChar(','):
+        if uilocale.decimalPoint() == ',':
             join = '; '
-        return join.join( [unicode(uilocale.toString(x)) for x in self.val] )
+        return join.join( [floattostring(x) for x in self.val] )
 
     def fromText(self, text):
         """Convert from a, b, c or a b c."""
 
         # don't use commas if it is the decimal separator
         splitre = r'[\t\n, ]+'
-        if uilocale.decimalPoint() == qt4.QChar(','):
+        if uilocale.decimalPoint() == ',':
             splitre = r'[\t\n; ]+'
 
         out = []
@@ -1057,7 +1067,7 @@ class WidgetPath(Str):
         it's the one.
 
         Returns None if setting is blank
-        InvalidType is raised if there's a problem
+        utils.InvalidType is raised if there's a problem
         """
 
         # this is a bit of a hack, so we don't have to pass a value
@@ -1081,7 +1091,7 @@ class WidgetPath(Str):
         try:
             widget = widget.document.resolve(widget, val)
         except ValueError:
-            raise InvalidType
+            raise utils.InvalidType
 
         # check the widget against the list of allowed types if given
         if self.allowedwidgets is not None:
@@ -1090,7 +1100,7 @@ class WidgetPath(Str):
                 if isinstance(widget, c):
                     allowed = True
             if not allowed:
-                raise InvalidType
+                raise utils.InvalidType
         
         return widget
 
@@ -1138,16 +1148,16 @@ class Strings(Setting):
         ('ds1','ds2'...)
         """
 
-        if isinstance(val, basestring):
+        if isinstance(val, cbasestr):
             return (val, )
 
         if type(val) not in (list, tuple):
-            raise InvalidType
+            raise utils.InvalidType
 
         # check each entry in the list is appropriate
         for ds in val:
-            if not isinstance(ds, basestring):
-                raise InvalidType
+            if not isinstance(ds, cbasestr):
+                raise utils.InvalidType
 
         return tuple(val)
         
@@ -1175,16 +1185,16 @@ class Datasets(Setting):
         ('ds1','ds2'...)
         """
 
-        if isinstance(val, basestring):
+        if isinstance(val, cbasestr):
             return (val, )
 
         if type(val) not in (list, tuple):
-            raise InvalidType
+            raise utils.InvalidType
 
         # check each entry in the list is appropriate
         for ds in val:
-            if not isinstance(ds, basestring):
-                raise InvalidType
+            if not isinstance(ds, cbasestr):
+                raise utils.InvalidType
 
         return tuple(val)
 
@@ -1221,7 +1231,7 @@ class DatasetExtended(Dataset):
         floats (numbers).
         """
 
-        if isinstance(val, basestring):
+        if isinstance(val, cbasestr):
             return val
         elif self.dimensions == 1:
             # list of numbers only allowed for 1d datasets
@@ -1232,17 +1242,17 @@ class DatasetExtended(Dataset):
                     return [float(x) for x in val]
                 except (TypeError, ValueError):
                     pass
-        raise InvalidType
+        raise utils.InvalidType
 
     def toText(self):
-        if isinstance(self.val, basestring):
+        if isinstance(self.val, cbasestr):
             return self.val
         else:
             # join based on , or ; depending on decimal point
             join = ', '
-            if uilocale.decimalPoint() == qt4.QChar(','):
+            if uilocale.decimalPoint() == ',':
                 join = '; '
-            return join.join( [ unicode(uilocale.toString(x))
+            return join.join( [ floattostring(x)
                                 for x in self.val ] )
 
     def fromText(self, text):
@@ -1255,7 +1265,7 @@ class DatasetExtended(Dataset):
 
         # split based on , or ; depending on decimal point
         splitre = r'[\t\n, ]+'
-        if uilocale.decimalPoint() == qt4.QChar(','):
+        if uilocale.decimalPoint() == ',':
             splitre = r'[\t\n; ]+'
 
         out = []
@@ -1271,7 +1281,7 @@ class DatasetExtended(Dataset):
 
     def getFloatArray(self, doc):
         """Get a numpy of values or None."""
-        if isinstance(self.val, basestring):
+        if isinstance(self.val, cbasestr):
             ds = doc.evalDatasetExpression(
                 self.val, datatype=self.datatype, dimensions=self.dimensions)
             if ds:
@@ -1284,7 +1294,7 @@ class DatasetExtended(Dataset):
 
     def isDataset(self, doc):
         """Is this setting a dataset?"""
-        return (isinstance(self.val, basestring) and
+        return (isinstance(self.val, cbasestr) and
                 doc.data.get(self.val))
 
     def isEmpty(self):
@@ -1293,7 +1303,7 @@ class DatasetExtended(Dataset):
 
     def getData(self, doc):
         """Return veusz dataset"""
-        if isinstance(self.val, basestring):
+        if isinstance(self.val, cbasestr):
             return doc.evalDatasetExpression(
                 self.val, datatype=self.datatype, dimensions=self.dimensions)
         else:
@@ -1301,29 +1311,36 @@ class DatasetExtended(Dataset):
                 self.val, self.datatype, self.dimensions)
 
 class DatasetOrStr(Dataset):
-    """Choose a dataset or enter a string."""
+    """Choose a dataset or enter a string.
+
+    Non string datasets are converted to string arrays using this.
+    """
 
     typename = 'dataset-or-str'
 
+    def __init__(self, name, val, **args):
+        Dataset.__init__(self, name, val, datatype='text', **args)
+
     def getData(self, doc, checknull=False):
         """Return either a list of strings, a single item list.
         If checknull then None is returned if blank
         """
         if doc:
             ds = doc.data.get(self.val)
-            if ds and ds.datatype == self.datatype:
-                return ds.data
+            if ds:
+                return doc.formatValsWithDatatypeToText(
+                    ds.data, ds.displaytype)
         if checknull and not self.val:
             return None
         else:
-            return [unicode(self.val)]
+            return [cstr(self.val)]
 
     def makeControl(self, *args):
-        # use string editor rather than drop down list
-        # need to write a custom control
-        return controls.DatasetOrString(self, self.getDocument(), self.dimensions,
-                                        self.datatype, *args)
-        #return controls.String(self, *args)
+        return controls.DatasetOrString(self, self.getDocument(), *args)
+
+    def copy(self):
+        """Make a setting which has its values copied from this one."""
+        return self._copyHelper((), (), {})
 
 class Color(ChoiceOrMore):
     """A color setting."""
@@ -1527,7 +1544,7 @@ class WidgetChoice(Str):
 
         # turn (object, level) pairs into object
         outdict = {}
-        for name, val in images.iteritems():
+        for name, val in citems(images):
             outdict[name] = val[0]
 
         return outdict
@@ -1593,20 +1610,20 @@ class LineSet(Setting):
         """
 
         if type(val) not in (list, tuple):
-            raise InvalidType
+            raise utils.InvalidType
 
         # check each entry in the list is appropriate
         for line in val:
             try:
                 style, width, color, hide = line
             except ValueError:
-                raise InvalidType
+                raise utils.InvalidType
 
-            if ( not isinstance(color, basestring) or
+            if ( not isinstance(color, cbasestr) or
                  not Distance.isDist(width) or
                  style not in LineStyle._linestyles or
                  type(hide) not in (int, bool) ):
-                raise InvalidType
+                raise utils.InvalidType
 
         return val
     
@@ -1660,20 +1677,20 @@ class FillSet(Setting):
         """
 
         if type(val) not in (list, tuple):
-            raise InvalidType
+            raise utils.InvalidType
 
         # check each entry in the list is appropriate
         for fill in val:
             try:
                 style, color, hide = fill[:3]
             except ValueError:
-                raise InvalidType
+                raise utils.InvalidType
 
-            if ( not isinstance(color, basestring) or
+            if ( not isinstance(color, cbasestr) or
                  style not in utils.extfillstyles or
                  type(hide) not in (int, bool) or
                  len(fill) not in (3, 10) ):
-                raise InvalidType
+                raise utils.InvalidType
 
         return val
     
@@ -1683,7 +1700,7 @@ class FillSet(Setting):
 
     def returnBrushExtended(self, row):
         """Return BrushExtended for the row."""
-        import collections
+        from . import collections
         s = collections.BrushExtended('tempbrush')
 
         if len(self.val) == 0:
@@ -1933,7 +1950,7 @@ class AxisBound(FloatOrAuto):
             mode = None
 
         v = self.val
-        if ( not isinstance(v, basestring) and v is not None and
+        if ( not isinstance(v, cbasestr) and v is not None and
              mode == 'datetime' ):
             return utils.dateFloatToString(v)
 
