@@ -19,18 +19,20 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
+from __future__ import division
 import math
 import re
 
 import numpy as N
-import veusz.qtall as qt4
 
-import points
+from ..compat import cbasestr, cstr
+from .. import qtall as qt4
+from . import points
 
 mmlsupport = True
 try:
-    import veusz.helpers.qtmml as qtmml
-    import veusz.helpers.recordpaint as recordpaint
+    from ..helpers import qtmml
+    from ..helpers import recordpaint
 except ImportError:
     mmlsupport = False
 
@@ -783,7 +785,7 @@ class PartFrac(Part):
         painter.drawLine(qt4.QPointF(initx,
                                      inity-height/2.),
                          qt4.QPointF(initx+max(self.widths),
-                                     inity-height/2))
+                                     inity-height/2.))
 
         painter.restore()
 
@@ -1101,12 +1103,12 @@ def makePartTree(partlist):
         if p == r'\\':
             lines.append( Part(itemlist) )
             itemlist = []
-        elif isinstance(p, basestring):
+        elif isinstance(p, cbasestr):
             if p in symbols:
                 addText(symbols[p])
             elif p in part_commands:
                 klass, numargs = part_commands[p]
-                if numargs == 1 and len(partlist) > i+1 and isinstance(partlist[i+1], basestring):
+                if numargs == 1 and len(partlist) > i+1 and isinstance(partlist[i+1], cbasestr):
                     # coerce a single argument to a partlist so that things
                     # like "A^\dagger" render correctly without needing
                     # curly brackets
@@ -1313,7 +1315,7 @@ class _StdRenderer(_Renderer):
                 # if want vertical centering, better to centre around middle
                 # of typical letter (i.e. where strike position is)
                 #totalheight = fm.strikeOutPos()*2
-                totalheight = fm.boundingRect(qt4.QChar('0')).height()
+                totalheight = fm.boundingRectChar('0').height()
             else:
                 # if top/bottom alignment, better to use maximum letter height
                 totalheight = fm.ascent()
@@ -1372,10 +1374,10 @@ class _MmlRenderer(_Renderer):
         self.mmldoc = doc = qtmml.QtMmlDocument()
         try:
             self.mmldoc.setContent(text)
-        except ValueError, e:
+        except ValueError as e:
             self.mmldoc = None
             self.error = ('Error interpreting MathML: %s\n' %
-                          unicode(e))
+                          cstr(e))
             return
 
         # this is pretty horrible :-(
