@@ -19,24 +19,25 @@
 """Widget to plot axes, and to handle conversion of coordinates to plot
 positions."""
 
-from itertools import izip
+from __future__ import division
+import re
 import numpy as N
 
-import veusz.qtall as qt4
-import veusz.document as document
-import veusz.setting as setting
-import veusz.utils as utils
+from ..compat import czip
+from .. import qtall as qt4
+from .. import document
+from .. import setting
+from .. import utils
 
-import widget
-import axisticks
-import controlgraph
+from . import widget
+from . import axisticks
+from . import controlgraph
 
 ###############################################################################
 
 def _(text, disambiguation=None, context='Axis'):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class MajorTick(setting.Line):
     '''Major tick settings.'''
@@ -320,8 +321,8 @@ class Axis(widget.Widget):
                pixmap='settings_axisminorgridlines' )
 
     @classmethod
-    def allowedParentTypes(self):
-        import graph, grid
+    def allowedParentTypes(klass):
+        from . import graph, grid
         return (graph.Graph, grid.Grid)
 
     @property
@@ -376,7 +377,7 @@ class Axis(widget.Widget):
             # this is ensured to be an Axis
             try:
                 widget = s.get('match').getReferredWidget()
-            except setting.InvalidType:
+            except utils.InvalidType:
                 widget = None
 
             # this looks valid + sanity checks
@@ -400,7 +401,7 @@ class Axis(widget.Widget):
         # tweak range to make sure things don't blow up further down the
         # line
         if ( abs(self.plottedrange[0] - self.plottedrange[1]) <
-             ( abs(self.plottedrange[0]) + abs(self.plottedrange[1]) )*1e-8 ):
+             ( abs(self.plottedrange[0]) + abs(self.plottedrange[1]) )*1e-12 ):
                self.plottedrange[1] = ( self.plottedrange[0] +
                                         max(1., self.plottedrange[0]*0.1) )
 
@@ -753,7 +754,7 @@ class Axis(widget.Widget):
             if None not in (labels, coords):
                 # convert coordinates to plotter coordinates
                 pcoords = self._graphToPlotter(coords)
-                for coord, pcoord, lab in izip(coords, pcoords, labels):
+                for coord, pcoord, lab in czip(coords, pcoords, labels):
                     # return labels that are within the plotted range
                     # of coordinates
                     if N.isfinite(coord) and (minval <= coord <= maxval):
@@ -804,7 +805,7 @@ class Axis(widget.Widget):
                 format = self.autoformat
 
             # generate positions and labels
-            for posn, tickval in izip(coordticks, tickvals):
+            for posn, tickval in czip(coordticks, tickvals):
                 text = utils.formatNumber(tickval*scale, format,
                                           locale=self.document.locale)
                 yield posn, text
@@ -1150,16 +1151,26 @@ class Axis(widget.Widget):
 
         if cgi.zoomed():
             # zoom axis scale
-            c1, c2 = self.plotterToGraphCoords(
-                cgi.maxposn, N.array([cgi.minzoom, cgi.maxzoom]))
+            # we convert a neighbouring pixel to see how we should
+            # round the text
+            c1, c2, c1delt, c2delt = self.plotterToGraphCoords(
+                cgi.maxposn, N.array([cgi.minzoom, cgi.maxzoom,
+                                      cgi.minzoom+1, cgi.maxzoom-1]))
             if c1 > c2:
                 c1, c2 = c2, c1
-            operations = (
-                document.OperationSettingSet(s.get('min'), float(c1)),
-                document.OperationSettingSet(s.get('max'), float(c2)),
-                )
+                c1delt, c2delt = c2delt, c1delt
+
+            ops = []
+            if s.min == 'Auto' or not N.allclose(c1, s.min, rtol=1e-8):
+                ops.append( document.OperationSettingSet(
+                        s.get('min'), utils.round2delt(c1, c1delt)) )
+            if s.max == 'Auto' or not N.allclose(c2, s.max, rtol=1e-8):
+                ops.append( document.OperationSettingSet(
+                        s.get('max'), utils.round2delt(c2, c2delt)) )
+
             self.document.applyOperation(
-                document.OperationMultiple(operations, descr=_('zoom axis')))
+                document.OperationMultiple(ops, descr=_('zoom axis')))
+
         elif cgi.moved():
             # move axis
             # convert positions to fractions
@@ -1174,13 +1185,18 @@ class Axis(widget.Widget):
                 minfrac, maxfrac = maxfrac, minfrac
 
             # update doc
-            operations = (
-                document.OperationSettingSet(s.get('lowerPosition'), minfrac),
-                document.OperationSettingSet(s.get('upperPosition'), maxfrac),
-                document.OperationSettingSet(s.get('otherPosition'), axisfrac),
-                )
+            ops = []
+            if s.lowerPosition != minfrac:
+                ops.append( document.OperationSettingSet(
+                        s.get('lowerPosition'), round(minfrac, 3)) )
+            if s.upperPosition != maxfrac:
+                ops.append( document.OperationSettingSet(
+                        s.get('upperPosition'), round(maxfrac, 3)) )
+            if s.otherPosition != axisfrac:
+                ops.append( document.OperationSettingSet(
+                        s.get('otherPosition'), round(axisfrac, 3)) )
             self.document.applyOperation(
-                document.OperationMultiple(operations, descr=_('adjust axis')))
+                document.OperationMultiple(ops, descr=_('adjust axis')))
 
 # allow the factory to instantiate an axis
 document.thefactory.register( Axis )
