@@ -19,10 +19,12 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ##############################################################################
 
+from __future__ import division
 import math
 import numpy as N
 
-import veusz.utils as utils
+from ..compat import crange
+from .. import utils
 
 """Algorithms for working with axis ticks.
 
@@ -80,7 +82,7 @@ class AxisTicks(AxisTicksBase):
                                       5.:  (0.5, 1., 2.5),
                                       2.5: (0.5,) }
     # just get the allowable majors
-    allowed_intervals_linear = allowed_minorintervals_linear.keys()
+    allowed_intervals_linear = sorted(allowed_minorintervals_linear.keys())
 
     # the allowed values we can increase by in log space
     # by default we increase by 10^3
@@ -138,8 +140,8 @@ class AxisTicks(AxisTicksBase):
                 if d <= maxextend:
                     maxval += d
 
-        # return (noticks, minbound, maxbound)
-        return ( self._tickNums(minval, maxval, delta), minval, maxval )
+        numticks = self._tickNums(minval, maxval, delta)
+        return (numticks, minval, maxval)
 
     def _calcLinearMinorTickValues(self, minval, maxval, interval, logstep,
                                    allowedintervals):
@@ -187,10 +189,10 @@ class AxisTicks(AxisTicksBase):
 
         ticks = []
         # iterate over range in log space
-        for i in xrange(alpha, beta+1):
+        for i in crange(alpha, beta+1):
             power = 10.**i
             # add ticks for values in correct range
-            for j in xrange(2, 10):
+            for j in crange(2, 10):
                 v = power*j
                 # blah log conversions mean we have to use 'fuzzy logic'
                 if ( math.fabs(v - minval)/v < 1e-6 or v > minval ) and \
