@@ -22,22 +22,22 @@ Contour plotting requires that the veusz_helpers package is installed,
 as a C routine (taken from matplotlib) is used to trace the contours.
 """
 
-from itertools import izip
+from __future__ import division, print_function
 import sys
 
-import veusz.qtall as qt4
+from ..compat import czip
+from .. import qtall as qt4
 import numpy as N
 
-import veusz.setting as setting
-import veusz.document as document
-import veusz.utils as utils
+from .. import setting
+from .. import document
+from .. import utils
 
-import plotters
+from . import plotters
 
 def _(text, disambiguation=None, context='Contour'):
     """Translate text."""
-    return unicode( 
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 def finitePoly(poly):
     """Remove non-finite coordinates from numpy arrays of coordinates."""
@@ -128,11 +128,11 @@ class Contour(plotters.GenericPlotter):
         # try to import contour helpers here
         Cntr = None
         try:
-            from veusz.helpers._nc_cntr import Cntr
+            from ..helpers._nc_cntr import Cntr
         except ImportError:
-            print >>sys.stderr,('WARNING: Veusz cannot import contour module\n'
+            print(('WARNING: Veusz cannot import contour module\n'
                                 'Please run python setup.py build\n'
-                                'Contour support is disabled')
+                                'Contour support is disabled'), file=sys.stderr)
             
         self.Cntr = Cntr
         # keep track of settings so we recalculate when necessary
@@ -217,7 +217,8 @@ class Contour(plotters.GenericPlotter):
 
         s.remove('key')
 
-    def _getUserDescription(self):
+    @property
+    def userdescription(self):
         """User friendly description."""
         s = self.settings
         out = []
@@ -229,7 +230,6 @@ class Contour(plotters.GenericPlotter):
         else:
             out.append('%(numLevels)i %(scaling)s levels (%(min)s to %(max)s)' % s)
         return ', '.join(out)
-    userdescription = property(_getUserDescription)
 
     def calculateLevels(self):
         """Calculate contour levels from data and settings.
@@ -324,7 +324,7 @@ class Contour(plotters.GenericPlotter):
         elif scaling == 'manual':
             drange = N.arange(1, num)
             out = [[]]
-            for conmin, conmax in izip(levels[:-1], levels[1:]):
+            for conmin, conmax in czip(levels[:-1], levels[1:]):
                 delta = (conmax-conmin) / num
                 out.append( conmin+drange*delta )
             slev = N.hstack(out)
@@ -413,8 +413,6 @@ class Contour(plotters.GenericPlotter):
     def dataDraw(self, painter, axes, posn, cliprect):
         """Draw the contours."""
 
-        s = self.settings
-
         # update contours if necessary
         if not self.checkContoursUpToDate():
             return
@@ -469,7 +467,7 @@ class Contour(plotters.GenericPlotter):
             # trace the polygons between the contours
             if len(s.Fills.fills) != 0 and len(levels) > 1 and not s.Fills.hide:
                 self._cachedpolygons = []
-                for level1, level2 in izip(levels[:-1], levels[1:]):
+                for level1, level2 in czip(levels[:-1], levels[1:]):
                     linelist = c.trace(level1, level2)
                     self._cachedpolygons.append( finitePoly(linelist) )
 
@@ -496,7 +494,7 @@ class Contour(plotters.GenericPlotter):
         descent = utils.FontMetrics(font, painter.device()).descent()
 
         # work out where text lies
-        half = len(xplt)/2
+        half = len(xplt) // 2
         hx, hy = xplt[half], yplt[half]
         r = utils.Renderer(painter, font, hx, hy, text, alignhorz=0,
                            alignvert=0, angle=0)
