@@ -18,27 +18,27 @@
 
 """For plotting xy points."""
 
-import veusz.qtall as qt4
-import itertools
+from __future__ import division
 import numpy as N
 
-import veusz.document as document
-import veusz.setting as setting
-import veusz.utils as utils
+from ..compat import czip
+from .. import qtall as qt4
+from .. import document
+from .. import setting
+from .. import utils
 
-import pickable
-from plotters import GenericPlotter
+from . import pickable
+from .plotters import GenericPlotter
 
 try:
-    import veusz.helpers.qtloops as qtloops
+    from ..helpers import qtloops
     hasqtloops = True
 except ImportError:
     hasqtloops = False
 
 def _(text, disambiguation=None, context='XY'):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # functions for plotting error bars
 # different styles are made up of combinations of these functions
@@ -141,7 +141,7 @@ def _errorBarsCurve(style, xmin, xmax, ymin, ymax, xplotter, yplotter,
         # non-filling brush
         painter.setBrush( qt4.QBrush() )
 
-        for xp, yp, xmn, ymn, xmx, ymx in itertools.izip(
+        for xp, yp, xmn, ymn, xmx, ymx in czip(
             xplotter, yplotter, xmin, ymin, xmax, ymax):
 
             # break up curve into four arcs (for asym error bars)
@@ -281,7 +281,7 @@ class PointPlotter(GenericPlotter):
                               usertext=_('Marker'), formatting=True), 0 )
         s.add( setting.DatasetOrStr('labels', '',
                                     descr=_('Dataset or string to label points'),
-                                    usertext=_('Labels'), datatype='text'), 5 )
+                                    usertext=_('Labels')), 5 )
         s.add( setting.DatasetExtended(
                 'scalePoints', '',
                 descr = _('Scale size of markers given by dataset, expression'
@@ -307,9 +307,9 @@ class PointPlotter(GenericPlotter):
                                   descr = _('Plot line settings'),
                                   usertext = _('Plot line')),
                pixmap = 'settings_plotline' )
-        s.add( setting.Line('MarkerLine',
-                            descr = _('Line around the marker settings'),
-                            usertext = _('Marker border')),
+        s.add( setting.MarkerLine('MarkerLine',
+                                  descr = _('Line around the marker settings'),
+                                  usertext = _('Marker border')),
                pixmap = 'settings_plotmarkerline' )
         s.add( MarkerFillBrush('MarkerFill',
                                descr = _('Marker fill settings'),
@@ -332,13 +332,13 @@ class PointPlotter(GenericPlotter):
                                   usertext=_('Label')),
                pixmap = 'settings_axislabel' )
 
-    def _getUserDescription(self):
+    @property
+    def userdescription(self):
         """User-friendly description."""
 
         s = self.settings
         return "x='%s', y='%s', marker='%s'" % (s.xData, s.yData,
                                                 s.marker)
-    userdescription = property(_getUserDescription)
 
     def _plotErrors(self, posn, painter, xplotter, yplotter,
                     axes, xdata, ydata, cliprect):
@@ -646,8 +646,8 @@ class PointPlotter(GenericPlotter):
         angle = lab.angle
 
         # iterate over each point and plot each label
-        for x, y, t in itertools.izip(xplotter+deltax, yplotter+deltay,
-                                      textvals):
+        for x, y, t in czip(xplotter+deltax, yplotter+deltay,
+                            textvals):
             utils.Renderer( painter, font, x, y, t,
                             alignhorz, alignvert, angle ).render()
 
@@ -729,7 +729,7 @@ class PointPlotter(GenericPlotter):
         # as datapoints
         if text:
             length = min( len(xv.data), len(yv.data) )
-            text = text*(length / len(text)) + text[:length % len(text)]
+            text = text*(length // len(text)) + text[:length % len(text)]
 
         # loop over chopped up values
         for xvals, yvals, tvals, ptvals, cvals in (
@@ -811,7 +811,8 @@ class PointPlotter(GenericPlotter):
                 # actually plot datapoints
                 utils.plotMarkers(painter, xplt, yplt, s.marker, markersize,
                                   scaling=scaling, clip=cliprect,
-                                  cmap=cmap, colorvals=colorvals)
+                                  cmap=cmap, colorvals=colorvals,
+                                  scaleline=s.MarkerLine.scaleLine)
 
             # finally plot any labels
             if tvals and not s.Label.hide:
