@@ -19,43 +19,45 @@
 
 """Implements the main window of the application."""
 
+from __future__ import division
 import os.path
 import sys
 import traceback
 import glob
+import codecs
 
-import veusz.qtall as qt4
+from ..compat import citems, ckeys, cstr, cexec, cstrerror
+from .. import qtall as qt4
 
-import veusz.document as document
-import veusz.utils as utils
-import veusz.utils.vzdbus as vzdbus
-import veusz.setting as setting
-import veusz.plugins as plugins
+from .. import document
+from .. import utils
+from ..utils import vzdbus
+from .. import setting
+from .. import plugins
 
-import consolewindow
-import plotwindow
-import treeeditwindow
-from datanavigator import DataNavigatorWindow
+from . import consolewindow
+from . import plotwindow
+from . import treeeditwindow
+from .datanavigator import DataNavigatorWindow
 
-from veusz.dialogs.aboutdialog import AboutDialog
-from veusz.dialogs.reloaddata import ReloadData
-from veusz.dialogs.datacreate import DataCreateDialog
-from veusz.dialogs.datacreate2d import DataCreate2DDialog
-from veusz.dialogs.preferences import PreferencesDialog
-from veusz.dialogs.errorloading import ErrorLoadingDialog
-from veusz.dialogs.capturedialog import CaptureDialog
-from veusz.dialogs.stylesheet import StylesheetDialog
-from veusz.dialogs.custom import CustomDialog
-from veusz.dialogs.safetyimport import SafetyImportDialog
-from veusz.dialogs.histodata import HistoDataDialog
-from veusz.dialogs.plugin import handlePlugin
-import veusz.dialogs.importdialog as importdialog
-import veusz.dialogs.dataeditdialog as dataeditdialog
+from ..dialogs.aboutdialog import AboutDialog
+from ..dialogs.reloaddata import ReloadData
+from ..dialogs.datacreate import DataCreateDialog
+from ..dialogs.datacreate2d import DataCreate2DDialog
+from ..dialogs.preferences import PreferencesDialog
+from ..dialogs.errorloading import ErrorLoadingDialog
+from ..dialogs.capturedialog import CaptureDialog
+from ..dialogs.stylesheet import StylesheetDialog
+from ..dialogs.custom import CustomDialog
+from ..dialogs.safetyimport import SafetyImportDialog
+from ..dialogs.histodata import HistoDataDialog
+from ..dialogs.plugin import handlePlugin
+from ..dialogs import importdialog
+from ..dialogs import dataeditdialog
 
 def _(text, disambiguation=None, context='MainWindow'):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # shortcut to this
 setdb = setting.settingdb
@@ -73,12 +75,12 @@ class DBusWinInterface(vzdbus.Object):
     @vzdbus.method(dbus_interface=interface, out_signature='as')
     def GetActions(self):
         """Get list of actions which can be activated."""
-        return list(sorted(self.actions.keys()))
+        return sorted(ckeys(self.actions))
 
     @vzdbus.method(dbus_interface=interface, in_signature='s')
     def TriggerAction(self, action):
         """Activate action given."""
-        self.actions[unicode(action)].trigger()
+        self.actions[action].trigger()
 
 class MainWindow(qt4.QMainWindow):
     """ The main window class for the application."""
@@ -207,8 +209,7 @@ class MainWindow(qt4.QMainWindow):
                       self.slotUpdatePickerLabel )
 
         # disable save if already saved
-        self.connect( self.document, qt4.SIGNAL("sigModified"),
-                      self.slotModifiedDoc )
+        self.document.signalModified.connect(self.slotModifiedDoc)
         # if the treeeditwindow changes the page, change the plot window
         self.connect( self.treeedit, qt4.SIGNAL("sigPageChanged"),
                       self.plot.setPageNumber )
@@ -268,7 +269,7 @@ class MainWindow(qt4.QMainWindow):
             return []
         else:
             # get list of vsz files dropped
-            urls = [unicode(u.path()) for u in mime.urls()]
+            urls = [u.path() for u in mime.urls()]
             urls = [u for u in urls if os.path.splitext(u)[1] == '.vsz']
             return urls
 
@@ -289,11 +290,11 @@ class MainWindow(qt4.QMainWindow):
             try:
                 self.document.applyOperation(
                     document.OperationLoadStyleSheet(filename) )
-            except EnvironmentError, e:
+            except EnvironmentError as e:
                 qt4.QMessageBox.warning(
                     self, _("Error - Veusz"),
                     _("Unable to load default stylesheet '%s'\n\n%s") %
-                    (filename, utils.decodeDefault(e.strerror)))
+                    (filename, cstrerror(e)))
             else:
                 # reset any modified flag
                 self.document.setModified(False)
@@ -306,11 +307,11 @@ class MainWindow(qt4.QMainWindow):
             try:
                 self.document.applyOperation(
                     document.OperationLoadCustom(filename) )
-            except EnvironmentError, e:
+            except EnvironmentError as e:
                 qt4.QMessageBox.warning(
                     self, _("Error - Veusz"),
                     _("Unable to load custom definitions '%s'\n\n%s") %
-                    (filename, utils.decodeDefault(e.strerror)))
+                    (filename, cstrerror(e)))
             else:
                 # reset any modified flag
                 self.document.setModified(False)
@@ -780,7 +781,7 @@ class MainWindow(qt4.QMainWindow):
         """Show a Veusz tutorial."""
         if self.document.isBlank():
             # run the tutorial
-            from veusz.windows.tutorial import TutorialDock
+            from .tutorial import TutorialDock
             tutdock = TutorialDock(self.document, self, self)
             self.addDockWidget(qt4.Qt.RightDockWidgetArea, tutdock)
             tutdock.show()
@@ -882,12 +883,12 @@ class MainWindow(qt4.QMainWindow):
                 ofile = open(self.filename, 'w')
                 self.document.saveToFile(ofile)
                 self.updateStatusbar(_("Saved to %s") % self.filename)
-            except EnvironmentError, e:
+            except EnvironmentError as e:
                 qt4.QApplication.restoreOverrideCursor()
                 qt4.QMessageBox.critical(
                     self, _("Error - Veusz"),
                     _("Unable to save document as '%s'\n\n%s") %
-                    (self.filename, utils.decodeDefault(e.strerror)))
+                    (self.filename, cstrerror(e)))
             else:
                 # restore the cursor
                 qt4.QApplication.restoreOverrideCursor()
@@ -919,7 +920,7 @@ class MainWindow(qt4.QMainWindow):
             # save directory for next time
             self.dirname = fd.directory().absolutePath()
             # update the edit box
-            filename = unicode( fd.selectedFiles()[0] )
+            filename = fd.selectedFiles()[0]
             if os.path.splitext(filename)[1] == '':
                 filename += '.' + filetype
 
@@ -940,14 +941,14 @@ class MainWindow(qt4.QMainWindow):
             # save directory for next time
             self.dirname = fd.directory().absolutePath()
 
-            filename = unicode( fd.selectedFiles()[0] )
+            filename = fd.selectedFiles()[0]
             try:
                 open(filename)
-            except EnvironmentError, e:
+            except EnvironmentError as e:
                 qt4.QMessageBox.critical(
                     self, _("Error - Veusz"),
                     _("Unable to open '%s'\n\n%s") %
-                    (filename, utils.decodeDefault(e.strerror)))
+                    (filename, cstrerror(e)))
                 return None
             return filename
         return None
@@ -1020,26 +1021,41 @@ class MainWindow(qt4.QMainWindow):
 
         # read script
         try:
-            script = open(filename, 'rU').read()
-        except EnvironmentError, e:
+            script = codecs.open(filename, 'rU', encoding='utf8').read()
+        except EnvironmentError as e:
             qt4.QApplication.restoreOverrideCursor()
             qt4.QMessageBox.critical(
                 self, _("Error - Veusz"),
                 _("Cannot open document '%s'\n\n%s") %
-                (filename, utils.decodeDefault(e.strerror)))
+                (filename, cstrerror(e)))
             self.setupDefaultDoc()
             return
 
-        # check code for any security issues
-        ignore_unsafe = setting.transient_settings['unsafe_mode']
-        if not ignore_unsafe:
-            errors = utils.checkCode(script, securityonly=True)
-            if errors:
+        def errordialog(e):
+            # display error dialog if there is an error loading
+            qt4.QApplication.restoreOverrideCursor()
+            i = sys.exc_info()
+            backtrace = traceback.format_exception( *i )
+            d = ErrorLoadingDialog(self, filename, str(e), ''.join(backtrace))
+            d.exec_()
+
+        # compile script and check for security (if reqd)
+        unsafe = setting.transient_settings['unsafe_mode']
+        while True:
+            try:
+                compiled = utils.compileChecked(script, mode='exec', filename=filename,
+                                                ignoresecurity=unsafe)
+                break
+            except utils.SafeEvalException:
+                # ask the user whether to execute in unsafe mode
                 qt4.QApplication.restoreOverrideCursor()
                 if ( self._unsafeCmdMsgBox(self, filename).exec_() ==
                      qt4.QMessageBox.No ):
                     return
-                ignore_unsafe = True # allow unsafe veusz commands below
+                unsafe = True
+            except Exception as e:
+                errordialog(e)
+                return
 
         # set up environment to run script
         env = self.document.eval_context.copy()
@@ -1053,7 +1069,7 @@ class MainWindow(qt4.QMainWindow):
         env['Root'] = interface.Root
 
         # wrap "unsafe" commands with a message box to check the user
-        safenow = [ignore_unsafe]
+        safenow = [unsafe]
         def _unsafeCaller(func):
             def wrapped(*args, **argsk):
                 if not safenow[0]:
@@ -1082,18 +1098,12 @@ class MainWindow(qt4.QMainWindow):
 
         try:
             # actually run script text
-            exec script in env
-        except Exception, e:
+            cexec(compiled, env)
+        except Exception as e:
             # need to remember to restore stdout, stderr
             sys.stdout, sys.stderr = stdout, stderr
-
-            # display error dialog if there is an error loading
-            qt4.QApplication.restoreOverrideCursor()
             self.document.enableUpdates()
-            i = sys.exc_info()
-            backtrace = traceback.format_exception( *i )
-            d = ErrorLoadingDialog(self, filename, str(e), ''.join(backtrace))
-            d.exec_()
+            errordialog(e)
             return
 
         # need to remember to restore stdout, stderr
@@ -1233,7 +1243,7 @@ class MainWindow(qt4.QMainWindow):
             # show busy cursor
             qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
 
-            filename = unicode( fd.selectedFiles()[0] )
+            filename = fd.selectedFiles()[0]
 
             # Add a default extension if one isn't supplied
             # this is the extension without the dot
@@ -1256,11 +1266,11 @@ class MainWindow(qt4.QMainWindow):
 
             try:
                 export.export()
-            except (RuntimeError, EnvironmentError), e:
+            except (RuntimeError, EnvironmentError) as e:
                 if isinstance(e, EnvironmentError):
-                    msg = utils.decodeDefault(e.strerror)
+                    msg = cstrerror(e)
                 else:
-                    msg = unicode(e)
+                    msg = cstr(e)
 
                 qt4.QApplication.restoreOverrideCursor()
                 qt4.QMessageBox.critical(
@@ -1303,7 +1313,7 @@ class MainWindow(qt4.QMainWindow):
         if values:
             # construct comma separated text representing axis values
             valitems = []
-            for name, val in values.iteritems():
+            for name, val in citems(values):
                 valitems.append('%s=%#.4g' % (name, val))
             valitems.sort()
             self.axisvalueslabel.setText(', '.join(valitems))
