@@ -20,19 +20,21 @@
 """Window to edit the document using a tree, widget properties
 and formatting properties."""
 
-import veusz.qtall as qt4
+from __future__ import division
 
-import veusz.widgets as widgets
-import veusz.utils as utils
-import veusz.document as document
-import veusz.setting as setting
+from ..compat import crange, citems
+from .. import qtall as qt4
 
-from widgettree import WidgetTreeModel, WidgetTreeView
+from .. import widgets
+from .. import utils
+from .. import document
+from .. import setting
+
+from .widgettree import WidgetTreeModel, WidgetTreeView
 
 def _(text, disambiguation=None, context='TreeEditWindow'):
     """Translate text."""
-    return unicode( 
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class SettingsProxy(object):
     """Object to handle communication between widget/settings
@@ -303,7 +305,7 @@ class PropertyList(qt4.QWidget):
         self.showformatsettings = showformatsettings
 
         self.layout = qt4.QGridLayout(self)
-        self.layout.setSpacing( self.layout.spacing()/2 )
+        self.layout.setSpacing( self.layout.spacing()//2 )
         self.layout.setMargin(4)
         
         self.childlist = []
@@ -807,13 +809,16 @@ class TreeEditDock(qt4.QDockWidget):
 
         # check whether each button can have this widget
         # (or a parent) as parent
-        for wc, action in self.addslots.iteritems():
+        for wc, action in citems(self.addslots):
             w = selw
             while w is not None and not wc.willAllowParent(w):
                 w = w.parent
 
             self.vzactions['add.%s' % wc.typename].setEnabled(w is not None)
 
+        self.vzactions['add.axismenu'].setEnabled(
+            self.vzactions['add.axis'].isEnabled())
+
         # exclusive widgets
         nonorth = self.vzactions['add.nonorthpoint'].isEnabled()
         self.vzactions['add.nonorthpoint'].setVisible(nonorth)
@@ -1036,7 +1041,7 @@ class TreeEditDock(qt4.QDockWidget):
     def doInitialWidgetSelect(self):
         """Select a sensible initial widget."""
         w = self.document.basewidget
-        for i in xrange(2):
+        for i in crange(2):
             try:
                 c = w.children[0]
             except IndexError:
@@ -1070,7 +1075,7 @@ class TreeEditDock(qt4.QDockWidget):
         # find indices of widgets to be deleted - find one to select after
         indexes = [widgetlist.index(w) for w in widgets]
         if -1 in indexes:
-            raise RuntimeError, "Invalid widget in list of selected widgets"
+            raise RuntimeError("Invalid widget in list of selected widgets")
         minindex = min(indexes)
 
         # delete selected widget
@@ -1206,7 +1211,7 @@ class SettingLabel(qt4.QWidget):
         self.setFocusPolicy(qt4.Qt.StrongFocus)
 
         self.document = document
-        self.connect(document, qt4.SIGNAL('sigModified'), self.slotDocModified)
+        document.signalModified.connect(self.slotDocModified)
 
         self.setting = setting
         self.setnsproxy = setnsproxy
@@ -1231,7 +1236,7 @@ class SettingLabel(qt4.QWidget):
         self.inmenu = False
 
         # initialise settings
-        self.slotDocModified()
+        self.slotDocModified(True)
 
     def mouseReleaseEvent(self, event):
         """Emit clicked(pos) on mouse release."""
@@ -1248,7 +1253,12 @@ class SettingLabel(qt4.QWidget):
         else:
             return qt4.QWidget.keyReleaseEvent(self, event)
 
-    def slotDocModified(self):
+    # Mark as a qt slot. This fixes a bug where you get C/C++ object
+    # deleted messages when the document emits signalModified but this
+    # widget has been deleted. This can be reproduced by dragging a
+    # widget between two windows, then undoing.
+    @qt4.pyqtSlot(int)
+    def slotDocModified(self, ismodified):
         """If the document has been modified."""
 
         # update pixmap (e.g. link added/removed)
