@@ -17,16 +17,16 @@
 #    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 ###############################################################################
 
+from __future__ import division
 import os.path
 
-import veusz.qtall as qt4
-import veusz.utils as utils
-import veusz.setting as setting
+from .. import qtall as qt4
+from .. import utils
+from .. import setting
 
 def _(text, disambiguation=None, context="Tutorial"):
     """Translate text."""
-    return unicode( 
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class TutorialStep(qt4.QObject):
     def __init__(self, text, mainwin,
@@ -56,8 +56,7 @@ class TutorialStep(qt4.QObject):
 
         self.nextonsetting = nextonsetting
         if nextonsetting is not None:
-            self.connect( mainwin.document,
-                          qt4.SIGNAL('sigModified'), self.slotNextSetting )
+            mainwin.document.signalModified.connect(self.slotNextSetting)
 
         self.nextonselected = nextonselected
         if nextonselected is not None:
@@ -263,7 +262,7 @@ class FunctionFormatLine(TutorialStep):
     def __init__(self, mainwin):
 
         tb = mainwin.formatdock.tabwidget.tabBar()
-        label = qt4.QLabel("  ")
+        label = qt4.QLabel("  ", tb)
         tb.setTabButton(1, qt4.QTabBar.LeftSide, label)
 
         TutorialStep.__init__(
@@ -347,7 +346,7 @@ class DataImport(TutorialStep):
 
     def slotDialogShown(self, dialog):
         """Called when a dialog is opened in the main window."""
-        from veusz.dialogs.importdialog import ImportDialog
+        from ..dialogs.importdialog import ImportDialog
         if isinstance(dialog, ImportDialog):
             # make life easy by sticking in filename
             dialog.filenameedit.setText(
@@ -386,8 +385,7 @@ to the original file.</p>
 '''), mainwin,
             nextstep=DataImportDialog3,
             disablenext=True)
-        self.connect( mainwin.document,
-                      qt4.SIGNAL('sigModified'), self.slotDocModified )
+        mainwin.document.signalModified.connect(self.slotDocModified)
 
     def slotDocModified(self):
         if 'alpha' in self.mainwin.document.data:
@@ -412,7 +410,7 @@ box or reopen it later.</p>
         self.timer.start(200)
 
     def slotTimeout(self):
-        from veusz.dialogs.importdialog import ImportDialog
+        from ..dialogs.importdialog import ImportDialog
         closed = True
         for dialog in self.mainwin.dialogs:
             if isinstance(dialog, ImportDialog):
@@ -642,8 +640,7 @@ deletes it from the document.</p>
             flash=mainwin.treeedit.edittoolbar.widgetForAction(
                 mainwin.vzactions['edit.cut']),
             nextstep=AddGrid)
-        self.connect( mainwin.document,
-                      qt4.SIGNAL('sigModified'), self.slotCheckDelete )
+        mainwin.document.signalModified.connect(self.slotCheckDelete)
 
     def slotCheckDelete(self, *args):
         d = self.mainwin.document
@@ -787,8 +784,7 @@ class TutorialDock(qt4.QDockWidget):
 
         # timer for controlling flashing
         self.flashtimer = qt4.QTimer(self)
-        self.connect(self.flashtimer, qt4.SIGNAL('timeout()'),
-                     self.slotFlashTimeout)
+        self.flashtimer.timeout.connect(self.slotFlashTimeout)
         self.flash = self.oldflash = None
         self.flashon = False
         self.flashct = 0
@@ -831,6 +827,8 @@ class TutorialDock(qt4.QDockWidget):
                 'Close', qt4.QDialogButtonBox.ActionRole)
             self.connect(closeb, qt4.SIGNAL('clicked()'), self.close)
 
+    # work around C/C++ object deleted
+    @qt4.pyqtSlot()
     def slotFlashTimeout(self):
         '''Handle flashing of UI components.'''
 
