@@ -18,15 +18,15 @@
 
 """Contains a model and view for handling a tree of widgets."""
 
-import veusz.qtall as qt4
+from __future__ import division
+from .. import qtall as qt4
 
-import veusz.utils as utils
-import veusz.document as document
+from .. import utils
+from .. import document
 
 def _(text, disambiguation=None, context="WidgetTree"):
     """Translate text."""
-    return unicode( 
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 class WidgetTreeModel(qt4.QAbstractItemModel):
     """A model representing the widget tree structure.
@@ -39,8 +39,7 @@ class WidgetTreeModel(qt4.QAbstractItemModel):
 
         self.document = document
 
-        self.connect( self.document, qt4.SIGNAL("sigModified"),
-                      self.slotDocumentModified )
+        document.signalModified.connect(self.slotDocumentModified)
         self.connect( self.document, qt4.SIGNAL("sigWiped"),
                       self.slotDocumentModified )
 
@@ -62,28 +61,28 @@ class WidgetTreeModel(qt4.QAbstractItemModel):
 
         # why do we get passed invalid indicies? :-)
         if not index.isValid():
-            return qt4.QVariant()
+            return None
 
         column = index.column()
         obj = index.internalPointer()
 
-        if role == qt4.Qt.DisplayRole:
+        if role in (qt4.Qt.DisplayRole, qt4.Qt.EditRole):
             # return text for columns
             if column == 0:
-                return qt4.QVariant(obj.name)
+                return obj.name
             elif column == 1:
-                return qt4.QVariant(obj.typename)
+                return obj.typename
 
         elif role == qt4.Qt.DecorationRole:
             # return icon for first column
             if column == 0:
                 filename = 'button_%s' % obj.typename
-                return qt4.QVariant(utils.getIcon(filename))
+                return utils.getIcon(filename)
 
         elif role == qt4.Qt.ToolTipRole:
             # provide tool tip showing description
             if obj.userdescription:
-                return qt4.QVariant(obj.userdescription)
+                return obj.userdescription
 
         elif role == qt4.Qt.TextColorRole:
             # show disabled looking text if object or any parent is hidden
@@ -97,17 +96,16 @@ class WidgetTreeModel(qt4.QAbstractItemModel):
 
             # return brush for hidden widget text, based on disabled text
             if hidden:
-                return qt4.QVariant(qt4.QPalette().brush(qt4.QPalette.Disabled,
-                                                         qt4.QPalette.Text))
+                return qt4.QPalette().brush(qt4.QPalette.Disabled,
+                                            qt4.QPalette.Text)
 
         # return nothing
-        return qt4.QVariant()
+        return None
 
-    def setData(self, index, value, role):
+    def setData(self, index, name, role):
         """User renames object. This renames the widget."""
         
         widget = index.internalPointer()
-        name = unicode(value.toString())
 
         # check symbols in name
         if not utils.validateWidgetName(name):
@@ -146,9 +144,9 @@ class WidgetTreeModel(qt4.QAbstractItemModel):
         
         if orientation == qt4.Qt.Horizontal and role == qt4.Qt.DisplayRole:
             val = ('Name', 'Type')[section]
-            return qt4.QVariant(val)
+            return val
 
-        return qt4.QVariant()
+        return None
 
     def index(self, row, column, parent):
         """Construct an index for a child of parent."""
@@ -238,10 +236,11 @@ class WidgetTreeModel(qt4.QAbstractItemModel):
 
         if action == qt4.Qt.IgnoreAction:
             return True
-        if not mimedata.hasFormat(document.widgetmime):
+
+        data = document.getWidgetMime(mimedata)
+        if data is None:
             return False
 
-        data = str(mimedata.data(document.widgetmime))
         if parentindex.isValid():
             parent = self.getWidget(parentindex)
         else:
@@ -324,7 +323,7 @@ class WidgetTreeView(qt4.QTreeView):
 
         if index.isValid():
             parent = self.model().getWidget(index)
-            data = str(event.mimeData().data(document.widgetmime))
+            data = document.getWidgetMime(event.mimeData())
             if document.isMimeDropable(parent, data):
                 # move the widget!
                 parentpath = parent.path
